# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Mesh/CellMat.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Mesh/CellMat
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import set_array, check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Mesh.CellMat.add_cell import add_cell
except ImportError as error:
    add_cell = error

try:
    from ..Methods.Mesh.CellMat.get_connectivity import get_connectivity
except ImportError as error:
    get_connectivity = error

try:
    from ..Methods.Mesh.CellMat.get_point2cell import get_point2cell
except ImportError as error:
    get_point2cell = error

try:
    from ..Methods.Mesh.CellMat.is_exist import is_exist
except ImportError as error:
    is_exist = error


from numpy import array, array_equal
from ._check import InitUnKnowClassError
from .Interpolation import Interpolation


class CellMat(FrozenClass):
    """Define the connectivity under matricial format containing one type of element (example: only triangles with 3 nodes). """

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Mesh.CellMat.add_cell
    if isinstance(add_cell, ImportError):
        add_cell = property(
            fget=lambda x: raise_(
                ImportError("Can't use CellMat method add_cell: " + str(add_cell))
            )
        )
    else:
        add_cell = add_cell
    # cf Methods.Mesh.CellMat.get_connectivity
    if isinstance(get_connectivity, ImportError):
        get_connectivity = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use CellMat method get_connectivity: "
                    + str(get_connectivity)
                )
            )
        )
    else:
        get_connectivity = get_connectivity
    # cf Methods.Mesh.CellMat.get_point2cell
    if isinstance(get_point2cell, ImportError):
        get_point2cell = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use CellMat method get_point2cell: " + str(get_point2cell)
                )
            )
        )
    else:
        get_point2cell = get_point2cell
    # cf Methods.Mesh.CellMat.is_exist
    if isinstance(is_exist, ImportError):
        is_exist = property(
            fget=lambda x: raise_(
                ImportError("Can't use CellMat method is_exist: " + str(is_exist))
            )
        )
    else:
        is_exist = is_exist
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        connectivity=[],
        nb_cell=0,
        nb_pt_per_cell=0,
        indice=[],
        interpolation=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "connectivity" in list(init_dict.keys()):
                connectivity = init_dict["connectivity"]
            if "nb_cell" in list(init_dict.keys()):
                nb_cell = init_dict["nb_cell"]
            if "nb_pt_per_cell" in list(init_dict.keys()):
                nb_pt_per_cell = init_dict["nb_pt_per_cell"]
            if "indice" in list(init_dict.keys()):
                indice = init_dict["indice"]
            if "interpolation" in list(init_dict.keys()):
                interpolation = init_dict["interpolation"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.connectivity = connectivity
        self.nb_cell = nb_cell
        self.nb_pt_per_cell = nb_pt_per_cell
        self.indice = indice
        self.interpolation = interpolation

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        CellMat_str = ""
        if self.parent is None:
            CellMat_str += "parent = None " + linesep
        else:
            CellMat_str += "parent = " + str(type(self.parent)) + " object" + linesep
        CellMat_str += (
            "connectivity = "
            + linesep
            + str(self.connectivity).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        CellMat_str += "nb_cell = " + str(self.nb_cell) + linesep
        CellMat_str += "nb_pt_per_cell = " + str(self.nb_pt_per_cell) + linesep
        CellMat_str += (
            "indice = "
            + linesep
            + str(self.indice).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        if self.interpolation is not None:
            tmp = (
                self.interpolation.__str__()
                .replace(linesep, linesep + "\t")
                .rstrip("\t")
            )
            CellMat_str += "interpolation = " + tmp
        else:
            CellMat_str += "interpolation = None" + linesep + linesep
        return CellMat_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if not array_equal(other.connectivity, self.connectivity):
            return False
        if other.nb_cell != self.nb_cell:
            return False
        if other.nb_pt_per_cell != self.nb_pt_per_cell:
            return False
        if not array_equal(other.indice, self.indice):
            return False
        if other.interpolation != self.interpolation:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if not array_equal(other.connectivity, self.connectivity):
            diff_list.append(name + ".connectivity")
        if other._nb_cell != self._nb_cell:
            diff_list.append(name + ".nb_cell")
        if other._nb_pt_per_cell != self._nb_pt_per_cell:
            diff_list.append(name + ".nb_pt_per_cell")
        if not array_equal(other.indice, self.indice):
            diff_list.append(name + ".indice")
        if (other.interpolation is None and self.interpolation is not None) or (
            other.interpolation is not None and self.interpolation is None
        ):
            diff_list.append(name + ".interpolation None mismatch")
        elif self.interpolation is not None:
            diff_list.extend(
                self.interpolation.compare(
                    other.interpolation, name=name + ".interpolation"
                )
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.connectivity)
        S += getsizeof(self.nb_cell)
        S += getsizeof(self.nb_pt_per_cell)
        S += getsizeof(self.indice)
        S += getsizeof(self.interpolation)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        CellMat_dict = dict()
        if self.connectivity is None:
            CellMat_dict["connectivity"] = None
        else:
            CellMat_dict["connectivity"] = self.connectivity.tolist()
        CellMat_dict["nb_cell"] = self.nb_cell
        CellMat_dict["nb_pt_per_cell"] = self.nb_pt_per_cell
        if self.indice is None:
            CellMat_dict["indice"] = None
        else:
            CellMat_dict["indice"] = self.indice.tolist()
        if self.interpolation is None:
            CellMat_dict["interpolation"] = None
        else:
            CellMat_dict["interpolation"] = self.interpolation.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        CellMat_dict["__class__"] = "CellMat"
        return CellMat_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.connectivity = None
        self.nb_cell = None
        self.nb_pt_per_cell = None
        self.indice = None
        if self.interpolation is not None:
            self.interpolation._set_None()

    def _get_connectivity(self):
        """getter of connectivity"""
        return self._connectivity

    def _set_connectivity(self, value):
        """setter of connectivity"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("connectivity", value, "ndarray")
        self._connectivity = value

    connectivity = property(
        fget=_get_connectivity,
        fset=_set_connectivity,
        doc=u"""Matrix of connectivity for one element type

        :Type: ndarray
        """,
    )

    def _get_nb_cell(self):
        """getter of nb_cell"""
        return self._nb_cell

    def _set_nb_cell(self, value):
        """setter of nb_cell"""
        check_var("nb_cell", value, "int")
        self._nb_cell = value

    nb_cell = property(
        fget=_get_nb_cell,
        fset=_set_nb_cell,
        doc=u"""Total number of elements

        :Type: int
        """,
    )

    def _get_nb_pt_per_cell(self):
        """getter of nb_pt_per_cell"""
        return self._nb_pt_per_cell

    def _set_nb_pt_per_cell(self, value):
        """setter of nb_pt_per_cell"""
        check_var("nb_pt_per_cell", value, "int")
        self._nb_pt_per_cell = value

    nb_pt_per_cell = property(
        fget=_get_nb_pt_per_cell,
        fset=_set_nb_pt_per_cell,
        doc=u"""Define the number of node per element

        :Type: int
        """,
    )

    def _get_indice(self):
        """getter of indice"""
        return self._indice

    def _set_indice(self, value):
        """setter of indice"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("indice", value, "ndarray")
        self._indice = value

    indice = property(
        fget=_get_indice,
        fset=_set_indice,
        doc=u"""Element indices

        :Type: ndarray
        """,
    )

    def _get_interpolation(self):
        """getter of interpolation"""
        return self._interpolation

    def _set_interpolation(self, value):
        """setter of interpolation"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "interpolation"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Interpolation()
        check_var("interpolation", value, "Interpolation")
        self._interpolation = value

        if self._interpolation is not None:
            self._interpolation.parent = self

    interpolation = property(
        fget=_get_interpolation,
        fset=_set_interpolation,
        doc=u"""Define FEA interpolation

        :Type: Interpolation
        """,
    )



# -*- coding: utf-8 -*-
from ....Methods.Geometry.Arc1 import *


def check(self):
    """assert that the arc is correct (begin != end; radius !=0)

    Parameters
    ----------
    self : Arc1
        An Arc1 object

    Returns
    -------
    None

    Raises
    ------
    PointArc1Error
        The beginning point and the ending point of an Arc1
        can't be the same
    RadiusArc1Error
        An Arc1 can't have a null radius

    """

    if self.begin == self.end or (abs(self.begin) == 0 and abs(self.end) == 0):
        raise PointArc1Error(
            "The beginning point and the ending "
            + "point of an Arc1 can't be "
            + "the same"
        )
    if self.radius == 0:
        raise RadiusArc1Error("An Arc1 can't have a null radius")



# -*- coding: utf-8 -*-

from ....Methods.Simulation.Input import InputError
from ....Classes.PostFunction import PostFunction
from ....Classes.PostMethod import PostMethod


def run_single(self):
    """Run the simulation

    Parameters
    ----------
    self : Simu1
        A Simu1 object

    """
    logger = self.get_logger()

    if self.parent is None:
        raise InputError("ERROR: Simulation object must be inside an Output object")
    output = self.parent
    output.geo = self.machine.comp_output_geo()

    # Init the input of the first module
    self.input.gen_input()

    # Run the modules
    if self.elec is not None:
        self.elec.run()
    if self.mag is not None:
        self.mag.run()

    if self.force is not None:
        self.force.run()
    # if self.HT is not None:
    #     self.HT.run()
    if self.struct is not None:
        self.struct.run()
    # if self.ac is not None:
    #     self.ac.run()

    if self.loss is not None:
        self.loss.run()
    # Running postprocessings

    if self.postproc_list:
        logger.info("Running simulation postprocessings...")
        for postproc in self.postproc_list:
            postproc.run(output)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/VarLoadCurrent.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/VarLoadCurrent
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import set_array, check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .VarLoad import VarLoad

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.VarLoadCurrent.get_input_list import get_input_list
except ImportError as error:
    get_input_list = error

try:
    from ..Methods.Simulation.VarLoadCurrent.generate_simulation_list import (
        generate_simulation_list,
    )
except ImportError as error:
    generate_simulation_list = error

try:
    from ..Methods.Simulation.VarLoadCurrent.gen_datakeeper_list import (
        gen_datakeeper_list,
    )
except ImportError as error:
    gen_datakeeper_list = error

try:
    from ..Methods.Simulation.VarLoadCurrent.check_param import check_param
except ImportError as error:
    check_param = error

try:
    from ..Methods.Simulation.VarLoadCurrent.get_elec_datakeeper import (
        get_elec_datakeeper,
    )
except ImportError as error:
    get_elec_datakeeper = error


from numpy import array, array_equal
from ._check import InitUnKnowClassError
from .DataKeeper import DataKeeper
from .VarSimu import VarSimu
from .Post import Post


class VarLoadCurrent(VarLoad):
    """Generate a multisimulation with InputCurrent at variable operating point"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.VarLoadCurrent.get_input_list
    if isinstance(get_input_list, ImportError):
        get_input_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarLoadCurrent method get_input_list: "
                    + str(get_input_list)
                )
            )
        )
    else:
        get_input_list = get_input_list
    # cf Methods.Simulation.VarLoadCurrent.generate_simulation_list
    if isinstance(generate_simulation_list, ImportError):
        generate_simulation_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarLoadCurrent method generate_simulation_list: "
                    + str(generate_simulation_list)
                )
            )
        )
    else:
        generate_simulation_list = generate_simulation_list
    # cf Methods.Simulation.VarLoadCurrent.gen_datakeeper_list
    if isinstance(gen_datakeeper_list, ImportError):
        gen_datakeeper_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarLoadCurrent method gen_datakeeper_list: "
                    + str(gen_datakeeper_list)
                )
            )
        )
    else:
        gen_datakeeper_list = gen_datakeeper_list
    # cf Methods.Simulation.VarLoadCurrent.check_param
    if isinstance(check_param, ImportError):
        check_param = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarLoadCurrent method check_param: " + str(check_param)
                )
            )
        )
    else:
        check_param = check_param
    # cf Methods.Simulation.VarLoadCurrent.get_elec_datakeeper
    if isinstance(get_elec_datakeeper, ImportError):
        get_elec_datakeeper = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarLoadCurrent method get_elec_datakeeper: "
                    + str(get_elec_datakeeper)
                )
            )
        )
    else:
        get_elec_datakeeper = get_elec_datakeeper
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        OP_matrix=None,
        type_OP_matrix=0,
        is_torque=False,
        is_power=False,
        name="",
        desc="",
        datakeeper_list=-1,
        is_keep_all_output=False,
        stop_if_error=False,
        var_simu=None,
        nb_simu=0,
        is_reuse_femm_file=True,
        postproc_list=-1,
        pre_keeper_postproc_list=None,
        post_keeper_postproc_list=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "OP_matrix" in list(init_dict.keys()):
                OP_matrix = init_dict["OP_matrix"]
            if "type_OP_matrix" in list(init_dict.keys()):
                type_OP_matrix = init_dict["type_OP_matrix"]
            if "is_torque" in list(init_dict.keys()):
                is_torque = init_dict["is_torque"]
            if "is_power" in list(init_dict.keys()):
                is_power = init_dict["is_power"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "datakeeper_list" in list(init_dict.keys()):
                datakeeper_list = init_dict["datakeeper_list"]
            if "is_keep_all_output" in list(init_dict.keys()):
                is_keep_all_output = init_dict["is_keep_all_output"]
            if "stop_if_error" in list(init_dict.keys()):
                stop_if_error = init_dict["stop_if_error"]
            if "var_simu" in list(init_dict.keys()):
                var_simu = init_dict["var_simu"]
            if "nb_simu" in list(init_dict.keys()):
                nb_simu = init_dict["nb_simu"]
            if "is_reuse_femm_file" in list(init_dict.keys()):
                is_reuse_femm_file = init_dict["is_reuse_femm_file"]
            if "postproc_list" in list(init_dict.keys()):
                postproc_list = init_dict["postproc_list"]
            if "pre_keeper_postproc_list" in list(init_dict.keys()):
                pre_keeper_postproc_list = init_dict["pre_keeper_postproc_list"]
            if "post_keeper_postproc_list" in list(init_dict.keys()):
                post_keeper_postproc_list = init_dict["post_keeper_postproc_list"]
        # Set the properties (value check and convertion are done in setter)
        self.OP_matrix = OP_matrix
        self.type_OP_matrix = type_OP_matrix
        self.is_torque = is_torque
        self.is_power = is_power
        # Call VarLoad init
        super(VarLoadCurrent, self).__init__(
            name=name,
            desc=desc,
            datakeeper_list=datakeeper_list,
            is_keep_all_output=is_keep_all_output,
            stop_if_error=stop_if_error,
            var_simu=var_simu,
            nb_simu=nb_simu,
            is_reuse_femm_file=is_reuse_femm_file,
            postproc_list=postproc_list,
            pre_keeper_postproc_list=pre_keeper_postproc_list,
            post_keeper_postproc_list=post_keeper_postproc_list,
        )
        # The class is frozen (in VarLoad init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        VarLoadCurrent_str = ""
        # Get the properties inherited from VarLoad
        VarLoadCurrent_str += super(VarLoadCurrent, self).__str__()
        VarLoadCurrent_str += (
            "OP_matrix = "
            + linesep
            + str(self.OP_matrix).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        VarLoadCurrent_str += "type_OP_matrix = " + str(self.type_OP_matrix) + linesep
        VarLoadCurrent_str += "is_torque = " + str(self.is_torque) + linesep
        VarLoadCurrent_str += "is_power = " + str(self.is_power) + linesep
        return VarLoadCurrent_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from VarLoad
        if not super(VarLoadCurrent, self).__eq__(other):
            return False
        if not array_equal(other.OP_matrix, self.OP_matrix):
            return False
        if other.type_OP_matrix != self.type_OP_matrix:
            return False
        if other.is_torque != self.is_torque:
            return False
        if other.is_power != self.is_power:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from VarLoad
        diff_list.extend(super(VarLoadCurrent, self).compare(other, name=name))
        if not array_equal(other.OP_matrix, self.OP_matrix):
            diff_list.append(name + ".OP_matrix")
        if other._type_OP_matrix != self._type_OP_matrix:
            diff_list.append(name + ".type_OP_matrix")
        if other._is_torque != self._is_torque:
            diff_list.append(name + ".is_torque")
        if other._is_power != self._is_power:
            diff_list.append(name + ".is_power")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from VarLoad
        S += super(VarLoadCurrent, self).__sizeof__()
        S += getsizeof(self.OP_matrix)
        S += getsizeof(self.type_OP_matrix)
        S += getsizeof(self.is_torque)
        S += getsizeof(self.is_power)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from VarLoad
        VarLoadCurrent_dict = super(VarLoadCurrent, self).as_dict(**kwargs)
        if self.OP_matrix is None:
            VarLoadCurrent_dict["OP_matrix"] = None
        else:
            VarLoadCurrent_dict["OP_matrix"] = self.OP_matrix.tolist()
        VarLoadCurrent_dict["type_OP_matrix"] = self.type_OP_matrix
        VarLoadCurrent_dict["is_torque"] = self.is_torque
        VarLoadCurrent_dict["is_power"] = self.is_power
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        VarLoadCurrent_dict["__class__"] = "VarLoadCurrent"
        return VarLoadCurrent_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.OP_matrix = None
        self.type_OP_matrix = None
        self.is_torque = None
        self.is_power = None
        # Set to None the properties inherited from VarLoad
        super(VarLoadCurrent, self)._set_None()

    def _get_OP_matrix(self):
        """getter of OP_matrix"""
        return self._OP_matrix

    def _set_OP_matrix(self, value):
        """setter of OP_matrix"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("OP_matrix", value, "ndarray")
        self._OP_matrix = value

    OP_matrix = property(
        fget=_get_OP_matrix,
        fset=_set_OP_matrix,
        doc=u"""Operating point matrix (N0,I0,Phi0,T,P) or (N0,Id,Iq,T,P) 

        :Type: ndarray
        """,
    )

    def _get_type_OP_matrix(self):
        """getter of type_OP_matrix"""
        return self._type_OP_matrix

    def _set_type_OP_matrix(self, value):
        """setter of type_OP_matrix"""
        check_var("type_OP_matrix", value, "int", Vmin=0, Vmax=1)
        self._type_OP_matrix = value

    type_OP_matrix = property(
        fget=_get_type_OP_matrix,
        fset=_set_type_OP_matrix,
        doc=u"""Select which kind of OP_matrix is used 0: (N0,I0,Phi0,T,P), 1:(N0,Id,Iq,T,P) 

        :Type: int
        :min: 0
        :max: 1
        """,
    )

    def _get_is_torque(self):
        """getter of is_torque"""
        return self._is_torque

    def _set_is_torque(self, value):
        """setter of is_torque"""
        check_var("is_torque", value, "bool")
        self._is_torque = value

    is_torque = property(
        fget=_get_is_torque,
        fset=_set_is_torque,
        doc=u"""True if the Torque is defined in OP_matrix

        :Type: bool
        """,
    )

    def _get_is_power(self):
        """getter of is_power"""
        return self._is_power

    def _set_is_power(self, value):
        """setter of is_power"""
        check_var("is_power", value, "bool")
        self._is_power = value

    is_power = property(
        fget=_get_is_power,
        fset=_set_is_power,
        doc=u"""True if the Power is defined in OP_matrix

        :Type: bool
        """,
    )



# -*- coding: utf-8 -*-

from numpy import exp, pi, tan


def comp_W1(self):
    """Compute W1 (cf schematics)

    Parameters
    ----------
    self : HoleM52
        A HoleM52 object

    Returns
    -------
    W1: float
        cf schematics [m]

    """

    Rext = self.get_Rext()

    alpha = self.comp_alpha()

    # Angle between (P1,P2) and (0,P0) is slot_pitch /2
    # It is also the angle (P1,P2,S)
    hsp = pi / self.Zh  # Half Slot Pitch

    # Distance P1,P9
    D19 = ((Rext - self.H0) * exp(1j * alpha / 2)).imag * 2

    # S is the intersectioni between (P1,P9) and the parallel to x passing by P2
    D1S = tan(hsp) * (self.H1 - self.H2)

    return D19 / 2.0 - D1S - self.W0 / 2.0



# -*- coding: utf-8 -*-


def comp_surface_active(self):
    """Compute the Slot inner surface for winding

    Parameters
    ----------
    self : SlotW61
        A SlotW61 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    return 2 * (self.H2 - self.H3 - self.H4) * ((self.W1 - self.W2) / 2 - self.W3)



def setter_simu(simu, input_obj):
    """Set simulation input with input_obj

    Parameters
    ----------
    simu: Simulation
        Pyleecan Simulation
    input_obj: Input
        Object defining the starting point of the simulation
    """

    simu.input = input_obj



def get_xoutput_ref(self):
    """Return the reference XOutput (or Output) either from xoutput_ref or output_list

    Parameters
    ----------
    self : XOutput
        A XOutput object
    Returns
    -------
    xoutput_ref: XOutput
        reference XOutput (or Output) (if defined)
    """

    if self.xoutput_ref_index is not None:
        return self.output_list[self.xoutput_ref_index]
    else:
        return self.xoutput_ref



# -*- coding: utf-8 -*-

from numpy import sin, pi


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotM17
        A SlotM17 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    return pi * self.parent.Rext ** 2 / 2



# -*- coding: utf-8 -*-
from numpy import zeros, sqrt, pi, tile, isnan
from multiprocessing import cpu_count

from ....Functions.Electrical.coordinate_transformation import n2ab, ab2n


def comp_parameters(self, output):
    """Compute the parameters dict for the equivalent electrical circuit:
    resistance, inductance
    Parameters
    ----------
    self : EEC_PMSM
        an EEC_PMSM object
    output : Output
        an Output object
    """
    # get some machine parameters
    machine = output.simu.machine
    Zsr = machine.rotor.slot.Zs
    qsr = machine.rotor.winding.qs
    p = machine.rotor.winding.p

    xi = machine.stator.winding.comp_winding_factor()
    Ntspc = machine.stator.winding.comp_Ntspc()
    norm = (xi[0] * Ntspc) / (Zsr / 6)  # rotor - stator transformation factor

    self.parameters["norm"] = norm

    # get temperatures TODO remove/replace, since this is a temp. solution only
    Tws = 20 if "Tws" not in self.parameters else self.parameter["Tws"]
    Twr = 20 if "Twr" not in self.parameters else self.parameter["Twr"]

    # Parameters to compute only if they are not set
    if "Rs" not in self.parameters or self.parameters["Rs"] is None:
        self.parameters["Rs"] = machine.stator.comp_resistance_wind(T=Tws)

    if "Rr_norm" not in self.parameters or self.parameters["Rr_norm"] is None:
        # 3 phase equivalent rotor resistance
        Rr = machine.rotor.comp_resistance_wind(T=Twr, qs=3)
        self.parameters["Rr_norm"] = norm ** 2 * Rr

    if "slip" not in self.parameters or self.parameters["slip"] is None:
        zp = output.simu.machine.stator.get_pole_pair_number()
        Nr = output.elec.N0
        Ns = output.elec.felec / zp * 60
        self.parameters["slip"] = (Ns - Nr) / Ns
        # print(f"slip = {(Ns - Nr) / Ns}")

    # check if inductances have to be calculated
    is_comp_ind = False

    if "Lm" not in self.parameters or self.parameters["Lm"] is None:
        is_comp_ind = True

    if "Ls" not in self.parameters or self.parameters["Ls"] is None:
        is_comp_ind = True

    if "Lr_norm" not in self.parameters or self.parameters["Lr_norm"] is None:
        is_comp_ind = True

    if "Rfe" not in self.parameters:
        self.parameters["Rfe"] = None  # TODO calculate (or estimate at least)

    if is_comp_ind:
        # setup a MagFEMM simulation to get the parameters
        # TODO maybe use IndMagFEMM or FluxlinkageFEMM
        #      but for now they are not suitable so I utilize 'normal' MagFEMM simu
        from ....Classes.Simu1 import Simu1
        from ....Classes.InputCurrent import InputCurrent
        from ....Classes.MagFEMM import MagFEMM
        from ....Classes.Output import Output
        from ....Classes.ImportGenVectLin import ImportGenVectLin
        from ....Classes.ImportMatrixVal import ImportMatrixVal

        # set frequency, time and number of revolutions
        # TODO what will be the best settings to get a good average with min. samples
        if self.N0 is None and self.felec is None:
            N0 = 50 * 60 / p
            felec = 50
        elif self.N0 is None and self.felec is not None:
            N0 = self.felec * 60 / p
        elif self.N0 is not None and self.felec is None:
            N0 = self.N0
            felec = N0 / 60 * p
        else:
            N0 = self.N0
            felec = self.felec

        Nrev = self.Nrev

        T = Nrev / (N0 / 60)
        Ir = ImportMatrixVal(value=zeros((self.Nt_tot, qsr)))
        time = ImportGenVectLin(start=0, stop=T, num=self.Nt_tot, endpoint=False)

        # TODO estimate magnetizing current if self.I == None
        # TODO compute magnetizing curve as function of I

        # setup the simu object
        simu = Simu1(name="EEC_comp_parameter", machine=machine.copy())
        # Definition of the enforced output of the electrical module
        simu.input = InputCurrent(
            Is=None,
            Id_ref=self.I,
            Iq_ref=0,
            Ir=Ir,  # zero current for the rotor
            N0=N0,
            angle_rotor=None,  # Will be computed
            time=time,
            felec=felec,
        )

        # Definition of the magnetic simulation (no symmetry)
        nb_worker = self.nb_worker if self.nb_worker else cpu_count()

        simu.mag = MagFEMM(
            type_BH_stator=0,
            type_BH_rotor=0,
            is_periodicity_a=self.is_periodicity_a,
            is_periodicity_t=False,
            Kgeo_fineness=0.5,
            Kmesh_fineness=0.5,
            nb_worker=nb_worker,
        )
        simu.force = None
        simu.struct = None

        # --- compute the main inductance and stator stray inductance ---
        # set output and run first simulation
        out = Output(simu=simu)
        out.simu.run()

        # compute average rotor and stator fluxlinkage
        # TODO check wind_mat that the i-th bars is in the i-th slots
        Phi_s, Phi_r = _comp_flux_mean(self, out)

        self.parameters["Lm"] = (Phi_r * norm * Zsr / 3) / self.I
        self.parameters["Ls"] = (Phi_s - (Phi_r * norm * Zsr / 3)) / self.I

        # --- compute the main inductance and rotor stray inductance ---
        # set new output
        out = Output(simu=simu)

        # set current values
        Ir_ = zeros([self.Nt_tot, 2])
        Ir_[:, 0] = self.I * norm * sqrt(2)
        Ir = ab2n(Ir_, n=qsr // p)  # TODO no rotation for now

        Ir = ImportMatrixVal(value=tile(Ir, (1, p)))

        simu.input.Is = None
        simu.input.Id_ref = 0
        simu.input.Iq_ref = 0
        simu.input.Ir = Ir

        out.simu.run()

        # compute average rotor and stator fluxlinkage
        # TODO check wind_mat that the i-th bars is in the i-th slots
        Phi_s, Phi_r = _comp_flux_mean(self, out)

        self.parameters["Lm_"] = Phi_s / self.I
        self.parameters["Lr_norm"] = ((Phi_r * norm * Zsr / 3) - Phi_s) / self.I


def _comp_flux_mean(self, out):
    # TODO add fix for single time value
    # TODO add option to calculate RMS instead of mean fluxlinkage

    # some readability
    logger = self.get_logger()
    machine = out.simu.machine
    p = machine.rotor.winding.p
    qsr = machine.rotor.winding.qs
    sym, is_anti_per, _, _ = machine.comp_periodicity()

    # get the fluxlinkages
    Phi = out.mag.Phi_wind["Rotor_0"].get_along("time", "phase")["Phi_{wind}"]

    # reconstruct fluxlinkage in case of (anti) periodicity
    if out.simu.mag.is_periodicity_a:
        # reconstruct anti periodicity
        if is_anti_per:
            qsr_eff = qsr // (sym * (1 + is_anti_per))
            for ii in range(qsr_eff):
                if not all(isnan(Phi[:, ii + qsr_eff]).tolist()):
                    logger.warning(
                        f"{type(self).__name__}: "
                        + f"Rotor fluxlinkage of bar {ii + qsr_eff} will be overridden."
                    )
                Phi[:, ii + qsr_eff] = -Phi[:, ii]
        # reconstruct periodicity
        if sym != 1:
            qsr_eff = qsr // sym
            if not all(isnan(Phi[:, qsr_eff:]).tolist()):
                logger.warning(
                    f"{type(self).__name__}: "
                    + f"Rotor fluxlinkage of bar "
                    + "starting with {qsr_eff} will be overridden."
                )
            for ii in range(sym - 1):
                id0 = qsr_eff * (ii + 1)
                id1 = qsr_eff * (ii + 2)
                Phi[:, id0:id1] = Phi[:, :qsr_eff]

        # rescale
        Phi = Phi / (sym * (1 + is_anti_per))

    # compute mean value of periodic bar fluxlinkage
    Phi_ab = zeros([Phi.shape[0], 2])
    if (qsr % p) == 0:
        qsr_per_pole = qsr // p
        for ii in range(p):
            id0 = qsr_per_pole * ii
            id1 = qsr_per_pole * (ii + 1)
            Phi_ab += n2ab(Phi[:, id0:id1], n=qsr_per_pole) / p
    else:
        logger.warning(f"{type(self).__name__}: " + "Not Implemented Yet")

    # compute rotor and stator fluxlinkage
    Phi_r = abs(Phi_ab[:, 0] + 1j * Phi_ab[:, 1]).mean() / sqrt(2)
    Phi_ab = n2ab(out.mag.Phi_wind["Stator_0"].get_along("time", "phase")["Phi_{wind}"])
    Phi_s = abs(Phi_ab[:, 0] + 1j * Phi_ab[:, 1]).mean() / sqrt(2)

    return Phi_s, Phi_r



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Import/ImportMatrix.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Import/ImportMatrix
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Import import Import

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Import.ImportMatrix.edit_matrix import edit_matrix
except ImportError as error:
    edit_matrix = error


from ._check import InitUnKnowClassError


class ImportMatrix(Import):
    """Abstract class to Import/Generate 1D or D matrix"""

    VERSION = 1

    # cf Methods.Import.ImportMatrix.edit_matrix
    if isinstance(edit_matrix, ImportError):
        edit_matrix = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ImportMatrix method edit_matrix: " + str(edit_matrix)
                )
            )
        )
    else:
        edit_matrix = edit_matrix
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, is_transpose=False, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "is_transpose" in list(init_dict.keys()):
                is_transpose = init_dict["is_transpose"]
        # Set the properties (value check and convertion are done in setter)
        self.is_transpose = is_transpose
        # Call Import init
        super(ImportMatrix, self).__init__()
        # The class is frozen (in Import init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ImportMatrix_str = ""
        # Get the properties inherited from Import
        ImportMatrix_str += super(ImportMatrix, self).__str__()
        ImportMatrix_str += "is_transpose = " + str(self.is_transpose) + linesep
        return ImportMatrix_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Import
        if not super(ImportMatrix, self).__eq__(other):
            return False
        if other.is_transpose != self.is_transpose:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Import
        diff_list.extend(super(ImportMatrix, self).compare(other, name=name))
        if other._is_transpose != self._is_transpose:
            diff_list.append(name + ".is_transpose")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Import
        S += super(ImportMatrix, self).__sizeof__()
        S += getsizeof(self.is_transpose)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Import
        ImportMatrix_dict = super(ImportMatrix, self).as_dict(**kwargs)
        ImportMatrix_dict["is_transpose"] = self.is_transpose
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        ImportMatrix_dict["__class__"] = "ImportMatrix"
        return ImportMatrix_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.is_transpose = None
        # Set to None the properties inherited from Import
        super(ImportMatrix, self)._set_None()

    def _get_is_transpose(self):
        """getter of is_transpose"""
        return self._is_transpose

    def _set_is_transpose(self, value):
        """setter of is_transpose"""
        check_var("is_transpose", value, "bool")
        self._is_transpose = value

    is_transpose = property(
        fget=_get_is_transpose,
        fset=_set_is_transpose,
        doc=u"""1 to transpose the Imported/Generated matrix

        :Type: bool
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/OutMagFEMM.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/OutMagFEMM
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .OutInternal import OutInternal

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Output.OutMagFEMM.clean import clean
except ImportError as error:
    clean = error


from ._check import InitUnKnowClassError
from ._FEMMHandler import _FEMMHandler


class OutMagFEMM(OutInternal):
    """Class to store outputs related to MagFEMM magnetic model"""

    VERSION = 1

    # cf Methods.Output.OutMagFEMM.clean
    if isinstance(clean, ImportError):
        clean = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutMagFEMM method clean: " + str(clean))
            )
        )
    else:
        clean = clean
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, FEMM_dict=None, handler_list=-1, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "FEMM_dict" in list(init_dict.keys()):
                FEMM_dict = init_dict["FEMM_dict"]
            if "handler_list" in list(init_dict.keys()):
                handler_list = init_dict["handler_list"]
        # Set the properties (value check and convertion are done in setter)
        self.FEMM_dict = FEMM_dict
        self.handler_list = handler_list
        # Call OutInternal init
        super(OutMagFEMM, self).__init__()
        # The class is frozen (in OutInternal init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OutMagFEMM_str = ""
        # Get the properties inherited from OutInternal
        OutMagFEMM_str += super(OutMagFEMM, self).__str__()
        OutMagFEMM_str += "FEMM_dict = " + str(self.FEMM_dict) + linesep
        if len(self.handler_list) == 0:
            OutMagFEMM_str += "handler_list = []" + linesep
        for ii in range(len(self.handler_list)):
            tmp = (
                self.handler_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            OutMagFEMM_str += (
                "handler_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        return OutMagFEMM_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from OutInternal
        if not super(OutMagFEMM, self).__eq__(other):
            return False
        if other.FEMM_dict != self.FEMM_dict:
            return False
        if other.handler_list != self.handler_list:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from OutInternal
        diff_list.extend(super(OutMagFEMM, self).compare(other, name=name))
        if other._FEMM_dict != self._FEMM_dict:
            diff_list.append(name + ".FEMM_dict")
        if (other.handler_list is None and self.handler_list is not None) or (
            other.handler_list is not None and self.handler_list is None
        ):
            diff_list.append(name + ".handler_list None mismatch")
        elif self.handler_list is None:
            pass
        elif len(other.handler_list) != len(self.handler_list):
            diff_list.append("len(" + name + ".handler_list)")
        else:
            for ii in range(len(other.handler_list)):
                diff_list.extend(
                    self.handler_list[ii].compare(
                        other.handler_list[ii],
                        name=name + ".handler_list[" + str(ii) + "]",
                    )
                )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from OutInternal
        S += super(OutMagFEMM, self).__sizeof__()
        if self.FEMM_dict is not None:
            for key, value in self.FEMM_dict.items():
                S += getsizeof(value) + getsizeof(key)
        if self.handler_list is not None:
            for value in self.handler_list:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from OutInternal
        OutMagFEMM_dict = super(OutMagFEMM, self).as_dict(**kwargs)
        OutMagFEMM_dict["FEMM_dict"] = (
            self.FEMM_dict.copy() if self.FEMM_dict is not None else None
        )
        if self.handler_list is None:
            OutMagFEMM_dict["handler_list"] = None
        else:
            OutMagFEMM_dict["handler_list"] = list()
            for obj in self.handler_list:
                if obj is not None:
                    OutMagFEMM_dict["handler_list"].append(obj.as_dict(**kwargs))
                else:
                    OutMagFEMM_dict["handler_list"].append(None)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        OutMagFEMM_dict["__class__"] = "OutMagFEMM"
        return OutMagFEMM_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.FEMM_dict = None
        self.handler_list = None
        # Set to None the properties inherited from OutInternal
        super(OutMagFEMM, self)._set_None()

    def _get_FEMM_dict(self):
        """getter of FEMM_dict"""
        return self._FEMM_dict

    def _set_FEMM_dict(self, value):
        """setter of FEMM_dict"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("FEMM_dict", value, "dict")
        self._FEMM_dict = value

    FEMM_dict = property(
        fget=_get_FEMM_dict,
        fset=_set_FEMM_dict,
        doc=u"""Dictionnary containing the main FEMM parameters

        :Type: dict
        """,
    )

    def _get_handler_list(self):
        """getter of handler_list"""
        if self._handler_list is not None:
            for obj in self._handler_list:
                if obj is not None:
                    obj.parent = self
        return self._handler_list

    def _set_handler_list(self, value):
        """setter of handler_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "handler_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("handler_list", value, "[_FEMMHandler]")
        self._handler_list = value

    handler_list = property(
        fget=_get_handler_list,
        fset=_set_handler_list,
        doc=u"""List of FEMM Handler (more than 1 if nb_worker >1)

        :Type: [_FEMMHandler]
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/Force.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/Force
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.Force.run import run
except ImportError as error:
    run = error

try:
    from ..Methods.Simulation.Force.comp_axes import comp_axes
except ImportError as error:
    comp_axes = error

try:
    from ..Methods.Simulation.Force.comp_AGSF_transfer import comp_AGSF_transfer
except ImportError as error:
    comp_AGSF_transfer = error


from ._check import InitUnKnowClassError


class Force(FrozenClass):
    """Forces module abstract object"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.Force.run
    if isinstance(run, ImportError):
        run = property(
            fget=lambda x: raise_(
                ImportError("Can't use Force method run: " + str(run))
            )
        )
    else:
        run = run
    # cf Methods.Simulation.Force.comp_axes
    if isinstance(comp_axes, ImportError):
        comp_axes = property(
            fget=lambda x: raise_(
                ImportError("Can't use Force method comp_axes: " + str(comp_axes))
            )
        )
    else:
        comp_axes = comp_axes
    # cf Methods.Simulation.Force.comp_AGSF_transfer
    if isinstance(comp_AGSF_transfer, ImportError):
        comp_AGSF_transfer = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Force method comp_AGSF_transfer: "
                    + str(comp_AGSF_transfer)
                )
            )
        )
    else:
        comp_AGSF_transfer = comp_AGSF_transfer
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        is_periodicity_t=None,
        is_periodicity_a=None,
        is_agsf_transfer=False,
        max_wavenumber_transfer=None,
        Rsbo_enforced_transfer=None,
        logger_name="Pyleecan.Force",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "is_periodicity_t" in list(init_dict.keys()):
                is_periodicity_t = init_dict["is_periodicity_t"]
            if "is_periodicity_a" in list(init_dict.keys()):
                is_periodicity_a = init_dict["is_periodicity_a"]
            if "is_agsf_transfer" in list(init_dict.keys()):
                is_agsf_transfer = init_dict["is_agsf_transfer"]
            if "max_wavenumber_transfer" in list(init_dict.keys()):
                max_wavenumber_transfer = init_dict["max_wavenumber_transfer"]
            if "Rsbo_enforced_transfer" in list(init_dict.keys()):
                Rsbo_enforced_transfer = init_dict["Rsbo_enforced_transfer"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.is_periodicity_t = is_periodicity_t
        self.is_periodicity_a = is_periodicity_a
        self.is_agsf_transfer = is_agsf_transfer
        self.max_wavenumber_transfer = max_wavenumber_transfer
        self.Rsbo_enforced_transfer = Rsbo_enforced_transfer
        self.logger_name = logger_name

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Force_str = ""
        if self.parent is None:
            Force_str += "parent = None " + linesep
        else:
            Force_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Force_str += "is_periodicity_t = " + str(self.is_periodicity_t) + linesep
        Force_str += "is_periodicity_a = " + str(self.is_periodicity_a) + linesep
        Force_str += "is_agsf_transfer = " + str(self.is_agsf_transfer) + linesep
        Force_str += (
            "max_wavenumber_transfer = " + str(self.max_wavenumber_transfer) + linesep
        )
        Force_str += (
            "Rsbo_enforced_transfer = " + str(self.Rsbo_enforced_transfer) + linesep
        )
        Force_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        return Force_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.is_periodicity_t != self.is_periodicity_t:
            return False
        if other.is_periodicity_a != self.is_periodicity_a:
            return False
        if other.is_agsf_transfer != self.is_agsf_transfer:
            return False
        if other.max_wavenumber_transfer != self.max_wavenumber_transfer:
            return False
        if other.Rsbo_enforced_transfer != self.Rsbo_enforced_transfer:
            return False
        if other.logger_name != self.logger_name:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._is_periodicity_t != self._is_periodicity_t:
            diff_list.append(name + ".is_periodicity_t")
        if other._is_periodicity_a != self._is_periodicity_a:
            diff_list.append(name + ".is_periodicity_a")
        if other._is_agsf_transfer != self._is_agsf_transfer:
            diff_list.append(name + ".is_agsf_transfer")
        if other._max_wavenumber_transfer != self._max_wavenumber_transfer:
            diff_list.append(name + ".max_wavenumber_transfer")
        if other._Rsbo_enforced_transfer != self._Rsbo_enforced_transfer:
            diff_list.append(name + ".Rsbo_enforced_transfer")
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.is_periodicity_t)
        S += getsizeof(self.is_periodicity_a)
        S += getsizeof(self.is_agsf_transfer)
        S += getsizeof(self.max_wavenumber_transfer)
        S += getsizeof(self.Rsbo_enforced_transfer)
        S += getsizeof(self.logger_name)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Force_dict = dict()
        Force_dict["is_periodicity_t"] = self.is_periodicity_t
        Force_dict["is_periodicity_a"] = self.is_periodicity_a
        Force_dict["is_agsf_transfer"] = self.is_agsf_transfer
        Force_dict["max_wavenumber_transfer"] = self.max_wavenumber_transfer
        Force_dict["Rsbo_enforced_transfer"] = self.Rsbo_enforced_transfer
        Force_dict["logger_name"] = self.logger_name
        # The class name is added to the dict for deserialisation purpose
        Force_dict["__class__"] = "Force"
        return Force_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.is_periodicity_t = None
        self.is_periodicity_a = None
        self.is_agsf_transfer = None
        self.max_wavenumber_transfer = None
        self.Rsbo_enforced_transfer = None
        self.logger_name = None

    def _get_is_periodicity_t(self):
        """getter of is_periodicity_t"""
        return self._is_periodicity_t

    def _set_is_periodicity_t(self, value):
        """setter of is_periodicity_t"""
        check_var("is_periodicity_t", value, "bool")
        self._is_periodicity_t = value

    is_periodicity_t = property(
        fget=_get_is_periodicity_t,
        fset=_set_is_periodicity_t,
        doc=u"""True to compute only on one time periodicity (use periodicities defined in output.force.Time). If None, automatically calculated based on Magnetics periodicities.

        :Type: bool
        """,
    )

    def _get_is_periodicity_a(self):
        """getter of is_periodicity_a"""
        return self._is_periodicity_a

    def _set_is_periodicity_a(self, value):
        """setter of is_periodicity_a"""
        check_var("is_periodicity_a", value, "bool")
        self._is_periodicity_a = value

    is_periodicity_a = property(
        fget=_get_is_periodicity_a,
        fset=_set_is_periodicity_a,
        doc=u"""True to compute only on one angle periodicity (use periodicities defined in output.force.Angle). If None, automatically calculated based on Magnetics periodicities.

        :Type: bool
        """,
    )

    def _get_is_agsf_transfer(self):
        """getter of is_agsf_transfer"""
        return self._is_agsf_transfer

    def _set_is_agsf_transfer(self, value):
        """setter of is_agsf_transfer"""
        check_var("is_agsf_transfer", value, "bool")
        self._is_agsf_transfer = value

    is_agsf_transfer = property(
        fget=_get_is_agsf_transfer,
        fset=_set_is_agsf_transfer,
        doc=u"""True to compute the AGSF transfer from air-gap to stator bore radius.

        :Type: bool
        """,
    )

    def _get_max_wavenumber_transfer(self):
        """getter of max_wavenumber_transfer"""
        return self._max_wavenumber_transfer

    def _set_max_wavenumber_transfer(self, value):
        """setter of max_wavenumber_transfer"""
        check_var("max_wavenumber_transfer", value, "int")
        self._max_wavenumber_transfer = value

    max_wavenumber_transfer = property(
        fget=_get_max_wavenumber_transfer,
        fset=_set_max_wavenumber_transfer,
        doc=u"""Maximum value to apply agsf transfer (to be used with FEA to avoid numerical noise amplification)

        :Type: int
        """,
    )

    def _get_Rsbo_enforced_transfer(self):
        """getter of Rsbo_enforced_transfer"""
        return self._Rsbo_enforced_transfer

    def _set_Rsbo_enforced_transfer(self, value):
        """setter of Rsbo_enforced_transfer"""
        check_var("Rsbo_enforced_transfer", value, "float")
        self._Rsbo_enforced_transfer = value

    Rsbo_enforced_transfer = property(
        fget=_get_Rsbo_enforced_transfer,
        fset=_set_Rsbo_enforced_transfer,
        doc=u"""To enforce the value of the radius for AGSF transfer

        :Type: float
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-


def comp_surface_magnet_id(self, index):
    """Compute the surface of the hole magnet of the corresponding index

    Parameters
    ----------
    self : HoleUD
        A HoleUD object
    index : int
        Index of the magnet to compute the surface

    Returns
    -------
    Smag: float
        Surface of the Magnet [m**2]
    """

    if self.magnet_dict is None:
        self.magnet_dict = dict()

    label = "magnet_" + str(index)
    if label in self.magnet_dict:
        if self.magnet_dict[label] is None:  # Magnet disabled
            return 0
        else:
            # Find the corresponding surface
            mag_id = 0
            for surf in self.surf_list:
                if "HoleMagnet" in surf.label and mag_id == index:
                    return surf.comp_surface()
                elif "HoleMagnet" in surf.label:
                    mag_id += 1
            return 0



# -*- coding: utf-8 -*-


def comp_resistance_wind(self, T=20):
    """Compute the DC winding resistance per phase without skin effect

    Parameters
    ----------
    self : LamSlotWind
        a LamSlotWind object
    T : float
        mean winding temperature [°C], default value is 20°C

    Returns
    -------
    R : float
        DC winding resistance per phase without skin effect
    """

    Zs = self.slot.Zs
    Ntspc = self.winding.comp_Ntspc(Zs)
    # length of the stack including ventilation ducts
    L1vd = self.comp_length()

    # end-winding length on one side for a half-turn
    # TODO implement endwinding length (return Enforced endwinding length)
    Lew = self.winding.comp_length_endwinding()

    # average length of a lamination winding half-turn (one "go" conductor
    # without "return" conductor)
    Lwht = L1vd + 2 * Lew

    # Active surface of conductor
    Sact = self.winding.conductor.comp_surface_active()

    # temperature coefficient and electrical conductivity
    alpha = self.winding.conductor.cond_mat.elec.alpha
    rhow20 = self.winding.conductor.cond_mat.elec.rho
    rhow = rhow20 * (1 + alpha * (T - 20))

    # DC winding resistance per phase at specified temperature
    R = (1.0 / self.winding.Npcpp) * rhow * (Ntspc * 2 * Lwht) / (Sact)

    return R



# -*- coding: utf-8 -*-
"""File generated according to PHoleM51/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM51.Ui_PHoleM51 import Ui_PHoleM51


class Gen_PHoleM51(Ui_PHoleM51):
    def setupUi(self, PHoleM51):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PHoleM51.setupUi(self, PHoleM51)
        # Setup of in_W0
        txt = self.tr(u"""Hole bottom width""")
        self.in_W0.setWhatsThis(txt)
        self.in_W0.setToolTip(txt)

        # Setup of lf_W0
        self.lf_W0.validator().setBottom(0)
        txt = self.tr(u"""Hole bottom width""")
        self.lf_W0.setWhatsThis(txt)
        self.lf_W0.setToolTip(txt)

        # Setup of in_W1
        txt = self.tr(u"""Hole angular width""")
        self.in_W1.setWhatsThis(txt)
        self.in_W1.setToolTip(txt)

        # Setup of lf_W1
        self.lf_W1.validator().setBottom(0)
        txt = self.tr(u"""Hole angular width""")
        self.lf_W1.setWhatsThis(txt)
        self.lf_W1.setToolTip(txt)

        # Setup of in_W3
        txt = self.tr(u"""magnet_1 width""")
        self.in_W3.setWhatsThis(txt)
        self.in_W3.setToolTip(txt)

        # Setup of lf_W3
        self.lf_W3.validator().setBottom(0)
        txt = self.tr(u"""magnet_1 width""")
        self.lf_W3.setWhatsThis(txt)
        self.lf_W3.setToolTip(txt)

        # Setup of in_W4
        txt = self.tr(u"""magnet_2 position""")
        self.in_W4.setWhatsThis(txt)
        self.in_W4.setToolTip(txt)

        # Setup of lf_W4
        self.lf_W4.validator().setBottom(0)
        txt = self.tr(u"""magnet_2 position""")
        self.lf_W4.setWhatsThis(txt)
        self.lf_W4.setToolTip(txt)

        # Setup of in_W5
        txt = self.tr(u"""magnet_2 width""")
        self.in_W5.setWhatsThis(txt)
        self.in_W5.setToolTip(txt)

        # Setup of lf_W5
        self.lf_W5.validator().setBottom(0)
        txt = self.tr(u"""magnet_2 width""")
        self.lf_W5.setWhatsThis(txt)
        self.lf_W5.setToolTip(txt)

        # Setup of in_W6
        txt = self.tr(u"""magnet_0 position""")
        self.in_W6.setWhatsThis(txt)
        self.in_W6.setToolTip(txt)

        # Setup of lf_W6
        self.lf_W6.validator().setBottom(0)
        txt = self.tr(u"""magnet_0 position""")
        self.lf_W6.setWhatsThis(txt)
        self.lf_W6.setToolTip(txt)

        # Setup of in_W7
        txt = self.tr(u"""magnet_0 width""")
        self.in_W7.setWhatsThis(txt)
        self.in_W7.setToolTip(txt)

        # Setup of lf_W7
        self.lf_W7.validator().setBottom(0)
        txt = self.tr(u"""magnet_0 width""")
        self.lf_W7.setWhatsThis(txt)
        self.lf_W7.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Hole depth""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Hole depth""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)

        # Setup of in_H1
        txt = self.tr(u"""Distance from the lamination Bore""")
        self.in_H1.setWhatsThis(txt)
        self.in_H1.setToolTip(txt)

        # Setup of lf_H1
        self.lf_H1.validator().setBottom(0)
        txt = self.tr(u"""Distance from the lamination Bore""")
        self.lf_H1.setWhatsThis(txt)
        self.lf_H1.setToolTip(txt)

        # Setup of in_H2
        txt = self.tr(u"""Hole width""")
        self.in_H2.setWhatsThis(txt)
        self.in_H2.setToolTip(txt)

        # Setup of lf_H2
        self.lf_H2.validator().setBottom(0)
        txt = self.tr(u"""Hole width""")
        self.lf_H2.setWhatsThis(txt)
        self.lf_H2.setToolTip(txt)



# -*- coding: utf-8 -*-

# File generated according to PWSlotUD.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Dialog.DMachineSetup.SWSlot.WWSlotOut.WWSlotOut import WWSlotOut
from ......GUI.Tools.WPathSelector.WPathSelectorV import WPathSelectorV
from ......GUI.Tools.MPLCanvas import MPLCanvas2

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_PWSlotUD(object):
    def setupUi(self, PWSlotUD):
        if not PWSlotUD.objectName():
            PWSlotUD.setObjectName(u"PWSlotUD")
        PWSlotUD.resize(740, 440)
        PWSlotUD.setMinimumSize(QSize(740, 440))
        PWSlotUD.setMaximumSize(QSize(16777215, 16777215))
        self.horizontalLayout = QHBoxLayout(PWSlotUD)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.w_viewer = MPLCanvas2(PWSlotUD)
        self.w_viewer.setObjectName(u"w_viewer")

        self.horizontalLayout.addWidget(self.w_viewer)

        self.widget = QWidget(PWSlotUD)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout = QVBoxLayout(self.widget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.b_dxf = QPushButton(self.widget)
        self.b_dxf.setObjectName(u"b_dxf")

        self.verticalLayout.addWidget(self.b_dxf)

        self.w_path_json = WPathSelectorV(self.widget)
        self.w_path_json.setObjectName(u"w_path_json")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.w_path_json.sizePolicy().hasHeightForWidth())
        self.w_path_json.setSizePolicy(sizePolicy)

        self.verticalLayout.addWidget(self.w_path_json)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout.addItem(self.verticalSpacer)

        self.w_out = WWSlotOut(self.widget)
        self.w_out.setObjectName(u"w_out")

        self.verticalLayout.addWidget(self.w_out)

        self.horizontalLayout.addWidget(self.widget)

        self.retranslateUi(PWSlotUD)

        QMetaObject.connectSlotsByName(PWSlotUD)

    # setupUi

    def retranslateUi(self, PWSlotUD):
        PWSlotUD.setWindowTitle(QCoreApplication.translate("PWSlotUD", u"Form", None))
        self.b_dxf.setText(
            QCoreApplication.translate("PWSlotUD", u"Define Slot from DXF", None)
        )

    # retranslateUi



# -*- coding: utf-8 -*-

from ....Methods.Machine import MachineCheckError


def check(self):
    """Check that the Machine object is correct

    Parameters
    ----------
    self :
        A Machine object

    Returns
    -------
    None

    Raises
    _______
    MC_AbstractError
        Machine is an abstract class
    MC_StatorNotStator
        self.stator.is_stator must be True
    MC_RotorIsStator
        self.rotor.is_stator must be False
    MC_BothInternal
        self.rotor.is_internal and self.rotor.is_internal can't be both True
    MC_BothExternal
        self.rotor.is_internal and self.rotor.is_internal can't be both False
    MC_RotorDontFit
        The Rotor is too big to fit in the Stator
    MC_ShaftTooBig
        The Shaft is too big to fit in the Rotor
    MC_ShaftTooSmall
        The Shaft is too small to fit in the Rotor
    MC_MecAirgapError
        The Stator and the rotor don't fit because of magnet or short circuit ring
    """

    if type(self).__name__ == "Machine":
        raise MC_AbstractError("Machine is an abstract class")

    if not self.stator.is_stator:
        raise MC_StatorNotStator("self.stator.is_stator must be True")

    if self.rotor.is_stator:
        raise MC_RotorIsStator("self.rotor.is_stator must be False")

    if self.rotor.is_internal and self.stator.is_internal:
        raise MC_BothInternal(
            "self.rotor.is_internal and " "self.rotor.is_internal can't be both True"
        )

    if (not self.rotor.is_internal) and (not self.stator.is_internal):
        raise MC_BothExternal(
            "self.rotor.is_internal and " "self.rotor.is_internal can't be both False"
        )

    if self.rotor.is_internal:
        if self.rotor.Rext > self.stator.Rint:
            raise MC_RotorDontFit("The Rotor is too big to fit in the Stator")
        if self.shaft.Drsh / 2.0 > self.rotor.Rint:
            raise MC_ShaftTooBig("The Shaft is too big to fit in the Rotor")
        if self.shaft.Drsh / 2.0 < self.rotor.Rint:
            raise MC_ShaftTooSmall("The Shaft is too small to fit in the " "Rotor")
    else:
        if self.stator.Rext > self.rotor.Rint:
            raise MC_StatorDontFit("The Stator is too big to fit in the Rotor")

    if self.comp_width_airgap_mec() <= 0:
        raise MC_MecAirgapError(
            "The Stator and the rotor don't fit because "
            "of magnet or short circuit ring"
        )

    self.rotor.check()
    self.stator.check()


class MC_AbstractError(MachineCheckError):
    """ """

    pass


class MC_StatorNotStator(MachineCheckError):
    """ """

    pass


class MC_RotorIsStator(MachineCheckError):
    """ """

    pass


class MC_BothInternal(MachineCheckError):
    """ """

    pass


class MC_BothExternal(MachineCheckError):
    """ """

    pass


class MC_RotorDontFit(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    stator


    """

    pass


class MC_StatorDontFit(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    rotor


    """

    pass


class MC_ShaftTooBig(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    rotor


    """

    pass


class MC_ShaftTooSmall(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    rotor


    """

    pass


class MC_MecAirgapError(MachineCheckError):
    """ """

    pass



from logging import Logger, FileHandler, Formatter, INFO, NOTSET
from datetime import datetime
from ....Classes.OptiObjective import OptiObjective


class OptimizationAttributeError(Exception):
    """Class to Raise an error"""

    def __init__(self, message):
        self.message = message


def check_optimization_input(self):
    """Check optimization parameters before solving the problem

    Parameters
    ----------
    solver : Solver
        solver to perform the genetic algorithm with DEAP
    """

    logger = self.get_logger()

    # Check problem existence
    if self.problem == None:
        raise OptimizationAttributeError(
            "The problem has not been defined, please add a problem to OptiGenAlgNsga2Deap."
        )

    # Check population size
    if self.size_pop % 4 > 0:
        mess = "Change population size from {} to {} to fit with the tournament selection".format(
            self.size_pop, self.size_pop + (self.size_pop % 4)
        )
        self.size_pop += self.size_pop % 4

        logger.warning(mess)

    # Check the problem contains at least one design variable
    if self.problem.obj_func == None or (
        isinstance(self.problem.obj_func, dict) and len(self.problem.obj_func) == 0
    ):
        raise OptimizationAttributeError(
            "Optimization problem must contain at least one objective function"
        )

    else:
        for obj_func in self.problem.obj_func:
            if not isinstance(obj_func, OptiObjective):
                raise TypeError(
                    "Wrong obj_func type: OptiObjective expected, got {}".format(
                        type(obj_func).__name__
                    )
                )
            elif not callable(obj_func.keeper):
                mess = "The objective function '{}' is not callable, please define the attribute 'keeper'.".format(
                    obj_func.name
                )
                raise OptimizationAttributeError(mess)

        # Check if objectives and other datakeepers have different symbol
        if isinstance(self.problem.datakeeper_list, list):
            symbol_list = [of.symbol for of in self.problem.obj_func] + [
                dk.symbol for dk in self.problem.datakeeper_list
            ]
        else:
            symbol_list = [of.symbol for of in self.problem.obj_func]

        if len(symbol_list) != len(set(symbol_list)):
            mess = "Every objective function and datakeeper must have a unique symbol."
            raise OptimizationAttributeError(mess)

    # Check the problem contains at least one objective function
    if self.problem.design_var == None or (
        isinstance(self.problem.design_var, list) and len(self.problem.design_var) == 0
    ):
        raise OptimizationAttributeError(
            "Optimization problem must contain at least one design variable"
        )
    else:
        for design_var in self.problem.design_var:
            if design_var.type_var not in ["set", "interval"]:
                mess = 'The design variable \'{}\' has a wrong type_var got {} expected "set" or "interval".'.format(
                    design_var.name, design_var.type_var
                )
                raise OptimizationAttributeError(mess)
            elif design_var.symbol in [None, ""]:
                mess = "The design variable '{}' has no symbol.".format(design_var.name)
                raise OptimizationAttributeError(mess)
            elif not callable(design_var.get_value):
                mess = "OptiDesignVar '{}' get_value is not callable.".format(
                    design_var.name
                )
                raise OptimizationAttributeError(mess)
            elif not callable(design_var.setter):
                mess = "OptiDesignVar '{}' setter is not callable.".format(
                    design_var.name
                )
                raise OptimizationAttributeError(mess)

    # Check constraints type
    if self.problem.constraint != None:
        for cstr in self.problem.constraint:
            # Check type of constraint
            if cstr.type_const not in ["<=", "<", "==", "=", ">=", ">"]:
                mess = "The constraint '{}' has a wrong type: expected one of {} received '{}'.".format(
                    cstr.name, ["<=", "<", "==", "=", ">=", ">"], cstr.type_const
                )
                raise OptimizationAttributeError(mess)
            # Check getter
            elif not callable(cstr.get_variable):
                mess = (
                    "The constraint '{}' function get_variable is not callable.".format(
                        cstr.name
                    )
                )
                raise OptimizationAttributeError(mess)



# -*-- coding: utf-8 -*
from numpy import exp


def rotate(self, angle):
    """Rotate the surface

    Parameters
    ----------
    self : Trapeze
        a Trapeze Object

    angle : float
        angle for rotation [rad]


    Returns
    -------
    None
    """
    # check if the Trapeze is correct
    self.check()
    self.point_ref = self.point_ref * exp(1j * angle)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/OutMag.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/OutMag
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Output.OutMag.store import store
except ImportError as error:
    store = error

try:
    from ..Methods.Output.OutMag.clean import clean
except ImportError as error:
    clean = error

try:
    from ..Methods.Output.OutMag.comp_emf import comp_emf
except ImportError as error:
    comp_emf = error

try:
    from ..Methods.Output.OutMag.get_demag import get_demag
except ImportError as error:
    get_demag = error


from ._check import InitUnKnowClassError
from .MeshSolution import MeshSolution
from .OutInternal import OutInternal


class OutMag(FrozenClass):
    """Gather the magnetic module outputs"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Output.OutMag.store
    if isinstance(store, ImportError):
        store = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutMag method store: " + str(store))
            )
        )
    else:
        store = store
    # cf Methods.Output.OutMag.clean
    if isinstance(clean, ImportError):
        clean = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutMag method clean: " + str(clean))
            )
        )
    else:
        clean = clean
    # cf Methods.Output.OutMag.comp_emf
    if isinstance(comp_emf, ImportError):
        comp_emf = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutMag method comp_emf: " + str(comp_emf))
            )
        )
    else:
        comp_emf = comp_emf
    # cf Methods.Output.OutMag.get_demag
    if isinstance(get_demag, ImportError):
        get_demag = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutMag method get_demag: " + str(get_demag))
            )
        )
    else:
        get_demag = get_demag
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Time=None,
        Angle=None,
        B=None,
        Tem=None,
        Tem_av=None,
        Tem_rip_norm=None,
        Tem_rip_pp=None,
        Phi_wind_stator=None,
        Phi_wind=None,
        emf=None,
        meshsolution=-1,
        logger_name="Pyleecan.Magnetics",
        internal=None,
        Rag=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Time" in list(init_dict.keys()):
                Time = init_dict["Time"]
            if "Angle" in list(init_dict.keys()):
                Angle = init_dict["Angle"]
            if "B" in list(init_dict.keys()):
                B = init_dict["B"]
            if "Tem" in list(init_dict.keys()):
                Tem = init_dict["Tem"]
            if "Tem_av" in list(init_dict.keys()):
                Tem_av = init_dict["Tem_av"]
            if "Tem_rip_norm" in list(init_dict.keys()):
                Tem_rip_norm = init_dict["Tem_rip_norm"]
            if "Tem_rip_pp" in list(init_dict.keys()):
                Tem_rip_pp = init_dict["Tem_rip_pp"]
            if "Phi_wind_stator" in list(init_dict.keys()):
                Phi_wind_stator = init_dict["Phi_wind_stator"]
            if "Phi_wind" in list(init_dict.keys()):
                Phi_wind = init_dict["Phi_wind"]
            if "emf" in list(init_dict.keys()):
                emf = init_dict["emf"]
            if "meshsolution" in list(init_dict.keys()):
                meshsolution = init_dict["meshsolution"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "internal" in list(init_dict.keys()):
                internal = init_dict["internal"]
            if "Rag" in list(init_dict.keys()):
                Rag = init_dict["Rag"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.Time = Time
        self.Angle = Angle
        self.B = B
        self.Tem = Tem
        self.Tem_av = Tem_av
        self.Tem_rip_norm = Tem_rip_norm
        self.Tem_rip_pp = Tem_rip_pp
        self.Phi_wind_stator = Phi_wind_stator
        self.Phi_wind = Phi_wind
        self.emf = emf
        self.meshsolution = meshsolution
        self.logger_name = logger_name
        self.internal = internal
        self.Rag = Rag

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OutMag_str = ""
        if self.parent is None:
            OutMag_str += "parent = None " + linesep
        else:
            OutMag_str += "parent = " + str(type(self.parent)) + " object" + linesep
        OutMag_str += "Time = " + str(self.Time) + linesep + linesep
        OutMag_str += "Angle = " + str(self.Angle) + linesep + linesep
        OutMag_str += "B = " + str(self.B) + linesep + linesep
        OutMag_str += "Tem = " + str(self.Tem) + linesep + linesep
        OutMag_str += "Tem_av = " + str(self.Tem_av) + linesep
        OutMag_str += "Tem_rip_norm = " + str(self.Tem_rip_norm) + linesep
        OutMag_str += "Tem_rip_pp = " + str(self.Tem_rip_pp) + linesep
        OutMag_str += (
            "Phi_wind_stator = " + str(self.Phi_wind_stator) + linesep + linesep
        )
        OutMag_str += "Phi_wind = " + str(self.Phi_wind) + linesep + linesep
        OutMag_str += "emf = " + str(self.emf) + linesep + linesep
        if self.meshsolution is not None:
            tmp = (
                self.meshsolution.__str__()
                .replace(linesep, linesep + "\t")
                .rstrip("\t")
            )
            OutMag_str += "meshsolution = " + tmp
        else:
            OutMag_str += "meshsolution = None" + linesep + linesep
        OutMag_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        if self.internal is not None:
            tmp = self.internal.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            OutMag_str += "internal = " + tmp
        else:
            OutMag_str += "internal = None" + linesep + linesep
        OutMag_str += "Rag = " + str(self.Rag) + linesep
        return OutMag_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.Time != self.Time:
            return False
        if other.Angle != self.Angle:
            return False
        if other.B != self.B:
            return False
        if other.Tem != self.Tem:
            return False
        if other.Tem_av != self.Tem_av:
            return False
        if other.Tem_rip_norm != self.Tem_rip_norm:
            return False
        if other.Tem_rip_pp != self.Tem_rip_pp:
            return False
        if other.Phi_wind_stator != self.Phi_wind_stator:
            return False
        if other.Phi_wind != self.Phi_wind:
            return False
        if other.emf != self.emf:
            return False
        if other.meshsolution != self.meshsolution:
            return False
        if other.logger_name != self.logger_name:
            return False
        if other.internal != self.internal:
            return False
        if other.Rag != self.Rag:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.Time is None and self.Time is not None) or (
            other.Time is not None and self.Time is None
        ):
            diff_list.append(name + ".Time None mismatch")
        elif self.Time is not None:
            diff_list.extend(self.Time.compare(other.Time, name=name + ".Time"))
        if (other.Angle is None and self.Angle is not None) or (
            other.Angle is not None and self.Angle is None
        ):
            diff_list.append(name + ".Angle None mismatch")
        elif self.Angle is not None:
            diff_list.extend(self.Angle.compare(other.Angle, name=name + ".Angle"))
        if (other.B is None and self.B is not None) or (
            other.B is not None and self.B is None
        ):
            diff_list.append(name + ".B None mismatch")
        elif self.B is not None:
            diff_list.extend(self.B.compare(other.B, name=name + ".B"))
        if (other.Tem is None and self.Tem is not None) or (
            other.Tem is not None and self.Tem is None
        ):
            diff_list.append(name + ".Tem None mismatch")
        elif self.Tem is not None:
            diff_list.extend(self.Tem.compare(other.Tem, name=name + ".Tem"))
        if other._Tem_av != self._Tem_av:
            diff_list.append(name + ".Tem_av")
        if other._Tem_rip_norm != self._Tem_rip_norm:
            diff_list.append(name + ".Tem_rip_norm")
        if other._Tem_rip_pp != self._Tem_rip_pp:
            diff_list.append(name + ".Tem_rip_pp")
        if (other.Phi_wind_stator is None and self.Phi_wind_stator is not None) or (
            other.Phi_wind_stator is not None and self.Phi_wind_stator is None
        ):
            diff_list.append(name + ".Phi_wind_stator None mismatch")
        elif self.Phi_wind_stator is not None:
            diff_list.extend(
                self.Phi_wind_stator.compare(
                    other.Phi_wind_stator, name=name + ".Phi_wind_stator"
                )
            )
        if (other.Phi_wind is None and self.Phi_wind is not None) or (
            other.Phi_wind is not None and self.Phi_wind is None
        ):
            diff_list.append(name + ".Phi_wind None mismatch")
        elif self.Phi_wind is None:
            pass
        elif len(other.Phi_wind) != len(self.Phi_wind):
            diff_list.append("len(" + name + "Phi_wind)")
        else:
            for key in self.Phi_wind:
                diff_list.extend(
                    self.Phi_wind[key].compare(
                        other.Phi_wind[key], name=name + ".Phi_wind"
                    )
                )
        if (other.emf is None and self.emf is not None) or (
            other.emf is not None and self.emf is None
        ):
            diff_list.append(name + ".emf None mismatch")
        elif self.emf is not None:
            diff_list.extend(self.emf.compare(other.emf, name=name + ".emf"))
        if (other.meshsolution is None and self.meshsolution is not None) or (
            other.meshsolution is not None and self.meshsolution is None
        ):
            diff_list.append(name + ".meshsolution None mismatch")
        elif self.meshsolution is not None:
            diff_list.extend(
                self.meshsolution.compare(
                    other.meshsolution, name=name + ".meshsolution"
                )
            )
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        if (other.internal is None and self.internal is not None) or (
            other.internal is not None and self.internal is None
        ):
            diff_list.append(name + ".internal None mismatch")
        elif self.internal is not None:
            diff_list.extend(
                self.internal.compare(other.internal, name=name + ".internal")
            )
        if other._Rag != self._Rag:
            diff_list.append(name + ".Rag")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.Time)
        S += getsizeof(self.Angle)
        S += getsizeof(self.B)
        S += getsizeof(self.Tem)
        S += getsizeof(self.Tem_av)
        S += getsizeof(self.Tem_rip_norm)
        S += getsizeof(self.Tem_rip_pp)
        S += getsizeof(self.Phi_wind_stator)
        if self.Phi_wind is not None:
            for key, value in self.Phi_wind.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.emf)
        S += getsizeof(self.meshsolution)
        S += getsizeof(self.logger_name)
        S += getsizeof(self.internal)
        S += getsizeof(self.Rag)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        OutMag_dict = dict()
        if self.Time is None:
            OutMag_dict["Time"] = None
        else:
            OutMag_dict["Time"] = self.Time.as_dict()
        if self.Angle is None:
            OutMag_dict["Angle"] = None
        else:
            OutMag_dict["Angle"] = self.Angle.as_dict()
        if self.B is None:
            OutMag_dict["B"] = None
        else:
            OutMag_dict["B"] = self.B.as_dict()
        if self.Tem is None:
            OutMag_dict["Tem"] = None
        else:
            OutMag_dict["Tem"] = self.Tem.as_dict()
        OutMag_dict["Tem_av"] = self.Tem_av
        OutMag_dict["Tem_rip_norm"] = self.Tem_rip_norm
        OutMag_dict["Tem_rip_pp"] = self.Tem_rip_pp
        if self.Phi_wind_stator is None:
            OutMag_dict["Phi_wind_stator"] = None
        else:
            OutMag_dict["Phi_wind_stator"] = self.Phi_wind_stator.as_dict()
        if self.Phi_wind is None:
            OutMag_dict["Phi_wind"] = None
        else:
            OutMag_dict["Phi_wind"] = dict()
            for key, obj in self.Phi_wind.items():
                if obj is not None:
                    OutMag_dict["Phi_wind"][key] = obj.as_dict()
                else:
                    OutMag_dict["Phi_wind"][key] = None
        if self.emf is None:
            OutMag_dict["emf"] = None
        else:
            OutMag_dict["emf"] = self.emf.as_dict()
        if self.meshsolution is None:
            OutMag_dict["meshsolution"] = None
        else:
            OutMag_dict["meshsolution"] = self.meshsolution.as_dict(**kwargs)
        OutMag_dict["logger_name"] = self.logger_name
        if self.internal is None:
            OutMag_dict["internal"] = None
        else:
            OutMag_dict["internal"] = self.internal.as_dict(**kwargs)
        OutMag_dict["Rag"] = self.Rag
        # The class name is added to the dict for deserialisation purpose
        OutMag_dict["__class__"] = "OutMag"
        return OutMag_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Time = None
        self.Angle = None
        self.B = None
        self.Tem = None
        self.Tem_av = None
        self.Tem_rip_norm = None
        self.Tem_rip_pp = None
        self.Phi_wind_stator = None
        self.Phi_wind = None
        self.emf = None
        if self.meshsolution is not None:
            self.meshsolution._set_None()
        self.logger_name = None
        if self.internal is not None:
            self.internal._set_None()
        self.Rag = None

    def _get_Time(self):
        """getter of Time"""
        return self._Time

    def _set_Time(self, value):
        """setter of Time"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Time"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Data()
        check_var("Time", value, "Data")
        self._Time = value

    Time = property(
        fget=_get_Time,
        fset=_set_Time,
        doc=u"""Magnetic time Data object

        :Type: SciDataTool.Classes.DataND.Data
        """,
    )

    def _get_Angle(self):
        """getter of Angle"""
        return self._Angle

    def _set_Angle(self, value):
        """setter of Angle"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Angle"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Data()
        check_var("Angle", value, "Data")
        self._Angle = value

    Angle = property(
        fget=_get_Angle,
        fset=_set_Angle,
        doc=u"""Magnetic position Data object

        :Type: SciDataTool.Classes.DataND.Data
        """,
    )

    def _get_B(self):
        """getter of B"""
        return self._B

    def _set_B(self, value):
        """setter of B"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("SciDataTool.Classes", value.get("__class__"), "B")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = VectorField()
        check_var("B", value, "VectorField")
        self._B = value

    B = property(
        fget=_get_B,
        fset=_set_B,
        doc=u"""Airgap flux density VectorField object

        :Type: SciDataTool.Classes.VectorField.VectorField
        """,
    )

    def _get_Tem(self):
        """getter of Tem"""
        return self._Tem

    def _set_Tem(self, value):
        """setter of Tem"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Tem"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataND()
        check_var("Tem", value, "DataND")
        self._Tem = value

    Tem = property(
        fget=_get_Tem,
        fset=_set_Tem,
        doc=u"""Electromagnetic torque DataTime object

        :Type: SciDataTool.Classes.DataND.DataND
        """,
    )

    def _get_Tem_av(self):
        """getter of Tem_av"""
        return self._Tem_av

    def _set_Tem_av(self, value):
        """setter of Tem_av"""
        check_var("Tem_av", value, "float")
        self._Tem_av = value

    Tem_av = property(
        fget=_get_Tem_av,
        fset=_set_Tem_av,
        doc=u"""Average Electromagnetic torque

        :Type: float
        """,
    )

    def _get_Tem_rip_norm(self):
        """getter of Tem_rip_norm"""
        return self._Tem_rip_norm

    def _set_Tem_rip_norm(self, value):
        """setter of Tem_rip_norm"""
        check_var("Tem_rip_norm", value, "float")
        self._Tem_rip_norm = value

    Tem_rip_norm = property(
        fget=_get_Tem_rip_norm,
        fset=_set_Tem_rip_norm,
        doc=u"""Peak to Peak Torque ripple normalized according to average torque (None if average torque=0)

        :Type: float
        """,
    )

    def _get_Tem_rip_pp(self):
        """getter of Tem_rip_pp"""
        return self._Tem_rip_pp

    def _set_Tem_rip_pp(self, value):
        """setter of Tem_rip_pp"""
        check_var("Tem_rip_pp", value, "float")
        self._Tem_rip_pp = value

    Tem_rip_pp = property(
        fget=_get_Tem_rip_pp,
        fset=_set_Tem_rip_pp,
        doc=u"""Peak to Peak Torque ripple

        :Type: float
        """,
    )

    def _get_Phi_wind_stator(self):
        """getter of Phi_wind_stator"""
        return self._Phi_wind_stator

    def _set_Phi_wind_stator(self, value):
        """setter of Phi_wind_stator"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Phi_wind_stator"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataTime()
        check_var("Phi_wind_stator", value, "DataTime")
        self._Phi_wind_stator = value

    Phi_wind_stator = property(
        fget=_get_Phi_wind_stator,
        fset=_set_Phi_wind_stator,
        doc=u"""Stator winding flux DataTime object

        :Type: SciDataTool.Classes.DataTime.DataTime
        """,
    )

    def _get_Phi_wind(self):
        """getter of Phi_wind"""
        if self._Phi_wind is not None:
            for key, obj in self._Phi_wind.items():
                if obj is not None:
                    obj.parent = self
        return self._Phi_wind

    def _set_Phi_wind(self, value):
        """setter of Phi_wind"""
        if type(value) is dict:
            for key, obj in value.items():
                if type(obj) is dict:
                    class_obj = import_class(
                        "SciDataTool.Classes", obj.get("__class__"), "Phi_wind"
                    )
                    value[key] = class_obj(init_dict=obj)
        if type(value) is int and value == -1:
            value = dict()
        check_var("Phi_wind", value, "{DataTime}")
        self._Phi_wind = value

    Phi_wind = property(
        fget=_get_Phi_wind,
        fset=_set_Phi_wind,
        doc=u"""Dict of lamination winding fluxlinkage DataTime objects

        :Type: {SciDataTool.Classes.DataTime.DataTime}
        """,
    )

    def _get_emf(self):
        """getter of emf"""
        return self._emf

    def _set_emf(self, value):
        """setter of emf"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "emf"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataTime()
        check_var("emf", value, "DataTime")
        self._emf = value

    emf = property(
        fget=_get_emf,
        fset=_set_emf,
        doc=u"""Electromotive force DataTime object

        :Type: SciDataTool.Classes.DataTime.DataTime
        """,
    )

    def _get_meshsolution(self):
        """getter of meshsolution"""
        return self._meshsolution

    def _set_meshsolution(self, value):
        """setter of meshsolution"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "meshsolution"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = MeshSolution()
        check_var("meshsolution", value, "MeshSolution")
        self._meshsolution = value

        if self._meshsolution is not None:
            self._meshsolution.parent = self

    meshsolution = property(
        fget=_get_meshsolution,
        fset=_set_meshsolution,
        doc=u"""FEA software mesh and solution

        :Type: MeshSolution
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )

    def _get_internal(self):
        """getter of internal"""
        return self._internal

    def _set_internal(self, value):
        """setter of internal"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "internal"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutInternal()
        check_var("internal", value, "OutInternal")
        self._internal = value

        if self._internal is not None:
            self._internal.parent = self

    internal = property(
        fget=_get_internal,
        fset=_set_internal,
        doc=u"""OutInternal object containg outputs related to a specific model

        :Type: OutInternal
        """,
    )

    def _get_Rag(self):
        """getter of Rag"""
        return self._Rag

    def _set_Rag(self, value):
        """setter of Rag"""
        check_var("Rag", value, "float")
        self._Rag = value

    Rag = property(
        fget=_get_Rag,
        fset=_set_Rag,
        doc=u"""Radius value for air-gap computation

        :Type: float
        """,
    )



# -*- coding: utf-8 -*-

from .....Methods.Output.Output.getter import GetOutError


def get_BH_rotor(self):
    """Return the B(H) curve of the rotor material

    Parameters
    ----------
    self : Output
        an Output object

    Returns
    -------
    BH: numpy.ndarray
        B(H) values (two colums matrix: H and B(H))

    """

    # Already available => Return
    if self.geo.rotor.BH_curve is not None:
        return self.geo.rotor.BH_curve

    # Check if possible to get the BH curve
    if (
        self.simu is None
        or self.simu.machine is None
        or self.simu.machine.rotor is None
        or self.simu.machine.rotor.mat_type is None
    ):
        raise GetOutError(
            "Output.simu.machine.rotor.mat_type is not Set, can't get the B(H) curve"
        )
    mat_type = self.simu.machine.rotor.mat_type
    if mat_type.mag is None:
        raise GetOutError("rotor materials magnetic property is not set")

    # Compute and store the BH curve
    BH = mat_type.mag.get_BH()
    self.geo.rotor.BH_curve = BH

    return BH


class BHShapeError(Exception):
    """Raised when the BH curve has not the expected shape"""

    pass



# -*- coding: utf-8 -*-

from numpy import linspace, where, sin, pi


def get_data(self):
    """Generate the toothsaw vector

    Parameters
    ----------
    self : ImportGenToothSaw
        An ImportGenToothSaw object

    Returns
    -------
    vect: ndarray
        The generated toothsaw vector

    """

    time = linspace(start=0, stop=self.Tf, num=self.N, endpoint=False)

    T = 1 / self.f
    time = (time + self.Dt) % T

    if self.type_signal == 0:  # forward toothsaw carrier
        Y = (
            where(time <= 0.5 * T, 1, 0) * time
            + where(time > 0.5 * T, 1, 0) * (time - T)
        ) / (0.5 * T)
    elif self.type_signal == 1:  # backwards toothsaw carrier
        Y = -(
            where(time <= 0.5 * T, 1, 0) * time
            + where(time > 0.5 * T, 1, 0) * (time - T)
        ) / (0.5 * T)
    elif self.type_signal == 2:  # symmetrical toothsaw carrier
        t1 = T / 4
        t2 = T - t1
        Y = (
            where(time <= t1, 1, 0) * time / t1
            + where(time > t1, 1, 0)
            * where(time < t2, 1, 0)
            * (-time + 0.5 * T)
            / (-t1 + 0.5 * T)
            + where(time >= t2, 1, 0) * (time - T) / (T - t2)
        )

    return self.edit_matrix(self.A * Y)



# -*- coding: utf-8 -*-
"""File generated according to PCondType12/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWindCond.PCondType12.Ui_PCondType12 import (
    Ui_PCondType12,
)


class Gen_PCondType12(Ui_PCondType12):
    def setupUi(self, PCondType12):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PCondType12.setupUi(self, PCondType12)
        # Setup of in_Nwpc1
        txt = self.tr(
            u"""cf schematics, winding number of random wires (strands) in parallel per coil"""
        )
        self.in_Nwpc1.setWhatsThis(txt)
        self.in_Nwpc1.setToolTip(txt)

        # Setup of si_Nwpc1
        self.si_Nwpc1.setMinimum(1)
        self.si_Nwpc1.setMaximum(999999)
        txt = self.tr(
            u"""cf schematics, winding number of random wires (strands) in parallel per coil"""
        )
        self.si_Nwpc1.setWhatsThis(txt)
        self.si_Nwpc1.setToolTip(txt)

        # Setup of in_Wwire
        txt = self.tr(u"""cf schematics, single wire diameter without insulation [m]""")
        self.in_Wwire.setWhatsThis(txt)
        self.in_Wwire.setToolTip(txt)

        # Setup of lf_Wwire
        self.lf_Wwire.validator().setBottom(0)
        txt = self.tr(u"""cf schematics, single wire diameter without insulation [m]""")
        self.lf_Wwire.setWhatsThis(txt)
        self.lf_Wwire.setToolTip(txt)

        # Setup of in_Wins_wire
        txt = self.tr(
            u"""(advanced) cf schematics, winding strand insulation thickness [m]"""
        )
        self.in_Wins_wire.setWhatsThis(txt)
        self.in_Wins_wire.setToolTip(txt)

        # Setup of lf_Wins_wire
        self.lf_Wins_wire.validator().setBottom(0)
        txt = self.tr(
            u"""(advanced) cf schematics, winding strand insulation thickness [m]"""
        )
        self.lf_Wins_wire.setWhatsThis(txt)
        self.lf_Wins_wire.setToolTip(txt)

        # Setup of in_Wins_cond
        txt = self.tr(
            u"""(advanced) cf schematics, winding coil insulation diameter [m]"""
        )
        self.in_Wins_cond.setWhatsThis(txt)
        self.in_Wins_cond.setToolTip(txt)

        # Setup of lf_Wins_cond
        self.lf_Wins_cond.validator().setBottom(0)
        txt = self.tr(
            u"""(advanced) cf schematics, winding coil insulation diameter [m]"""
        )
        self.lf_Wins_cond.setWhatsThis(txt)
        self.lf_Wins_cond.setToolTip(txt)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Slot/HoleUD.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Slot/HoleUD
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .HoleMag import HoleMag

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Slot.HoleUD.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error

try:
    from ..Methods.Slot.HoleUD.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Slot.HoleUD.comp_surface_magnet_id import comp_surface_magnet_id
except ImportError as error:
    comp_surface_magnet_id = error

try:
    from ..Methods.Slot.HoleUD.has_magnet import has_magnet
except ImportError as error:
    has_magnet = error

try:
    from ..Methods.Slot.HoleUD.remove_magnet import remove_magnet
except ImportError as error:
    remove_magnet = error


from ._check import InitUnKnowClassError
from .Surface import Surface
from .Magnet import Magnet
from .Material import Material


class HoleUD(HoleMag):
    """User defined hole from list of surface"""

    VERSION = 1
    IS_SYMMETRICAL = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Slot.HoleUD.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use HoleUD method build_geometry: " + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # cf Methods.Slot.HoleUD.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use HoleUD method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Slot.HoleUD.comp_surface_magnet_id
    if isinstance(comp_surface_magnet_id, ImportError):
        comp_surface_magnet_id = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use HoleUD method comp_surface_magnet_id: "
                    + str(comp_surface_magnet_id)
                )
            )
        )
    else:
        comp_surface_magnet_id = comp_surface_magnet_id
    # cf Methods.Slot.HoleUD.has_magnet
    if isinstance(has_magnet, ImportError):
        has_magnet = property(
            fget=lambda x: raise_(
                ImportError("Can't use HoleUD method has_magnet: " + str(has_magnet))
            )
        )
    else:
        has_magnet = has_magnet
    # cf Methods.Slot.HoleUD.remove_magnet
    if isinstance(remove_magnet, ImportError):
        remove_magnet = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use HoleUD method remove_magnet: " + str(remove_magnet)
                )
            )
        )
    else:
        remove_magnet = remove_magnet
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        surf_list=-1,
        magnet_dict=-1,
        Zh=36,
        mat_void=-1,
        magnetization_dict_offset=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "surf_list" in list(init_dict.keys()):
                surf_list = init_dict["surf_list"]
            if "magnet_dict" in list(init_dict.keys()):
                magnet_dict = init_dict["magnet_dict"]
            if "Zh" in list(init_dict.keys()):
                Zh = init_dict["Zh"]
            if "mat_void" in list(init_dict.keys()):
                mat_void = init_dict["mat_void"]
            if "magnetization_dict_offset" in list(init_dict.keys()):
                magnetization_dict_offset = init_dict["magnetization_dict_offset"]
        # Set the properties (value check and convertion are done in setter)
        self.surf_list = surf_list
        self.magnet_dict = magnet_dict
        # Call HoleMag init
        super(HoleUD, self).__init__(
            Zh=Zh,
            mat_void=mat_void,
            magnetization_dict_offset=magnetization_dict_offset,
        )
        # The class is frozen (in HoleMag init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        HoleUD_str = ""
        # Get the properties inherited from HoleMag
        HoleUD_str += super(HoleUD, self).__str__()
        if len(self.surf_list) == 0:
            HoleUD_str += "surf_list = []" + linesep
        for ii in range(len(self.surf_list)):
            tmp = (
                self.surf_list[ii].__str__().replace(linesep, linesep + "\t") + linesep
            )
            HoleUD_str += "surf_list[" + str(ii) + "] =" + tmp + linesep + linesep
        if len(self.magnet_dict) == 0:
            HoleUD_str += "magnet_dict = dict()" + linesep
        for key, obj in self.magnet_dict.items():
            tmp = (
                self.magnet_dict[key].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            HoleUD_str += "magnet_dict[" + key + "] =" + tmp + linesep + linesep
        return HoleUD_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from HoleMag
        if not super(HoleUD, self).__eq__(other):
            return False
        if other.surf_list != self.surf_list:
            return False
        if other.magnet_dict != self.magnet_dict:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from HoleMag
        diff_list.extend(super(HoleUD, self).compare(other, name=name))
        if (other.surf_list is None and self.surf_list is not None) or (
            other.surf_list is not None and self.surf_list is None
        ):
            diff_list.append(name + ".surf_list None mismatch")
        elif self.surf_list is None:
            pass
        elif len(other.surf_list) != len(self.surf_list):
            diff_list.append("len(" + name + ".surf_list)")
        else:
            for ii in range(len(other.surf_list)):
                diff_list.extend(
                    self.surf_list[ii].compare(
                        other.surf_list[ii], name=name + ".surf_list[" + str(ii) + "]"
                    )
                )
        if (other.magnet_dict is None and self.magnet_dict is not None) or (
            other.magnet_dict is not None and self.magnet_dict is None
        ):
            diff_list.append(name + ".magnet_dict None mismatch")
        elif self.magnet_dict is None:
            pass
        elif len(other.magnet_dict) != len(self.magnet_dict):
            diff_list.append("len(" + name + "magnet_dict)")
        else:
            for key in self.magnet_dict:
                diff_list.extend(
                    self.magnet_dict[key].compare(
                        other.magnet_dict[key], name=name + ".magnet_dict"
                    )
                )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from HoleMag
        S += super(HoleUD, self).__sizeof__()
        if self.surf_list is not None:
            for value in self.surf_list:
                S += getsizeof(value)
        if self.magnet_dict is not None:
            for key, value in self.magnet_dict.items():
                S += getsizeof(value) + getsizeof(key)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from HoleMag
        HoleUD_dict = super(HoleUD, self).as_dict(**kwargs)
        if self.surf_list is None:
            HoleUD_dict["surf_list"] = None
        else:
            HoleUD_dict["surf_list"] = list()
            for obj in self.surf_list:
                if obj is not None:
                    HoleUD_dict["surf_list"].append(obj.as_dict(**kwargs))
                else:
                    HoleUD_dict["surf_list"].append(None)
        if self.magnet_dict is None:
            HoleUD_dict["magnet_dict"] = None
        else:
            HoleUD_dict["magnet_dict"] = dict()
            for key, obj in self.magnet_dict.items():
                if obj is not None:
                    HoleUD_dict["magnet_dict"][key] = obj.as_dict(**kwargs)
                else:
                    HoleUD_dict["magnet_dict"][key] = None
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        HoleUD_dict["__class__"] = "HoleUD"
        return HoleUD_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.surf_list = None
        self.magnet_dict = None
        # Set to None the properties inherited from HoleMag
        super(HoleUD, self)._set_None()

    def _get_surf_list(self):
        """getter of surf_list"""
        if self._surf_list is not None:
            for obj in self._surf_list:
                if obj is not None:
                    obj.parent = self
        return self._surf_list

    def _set_surf_list(self, value):
        """setter of surf_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "surf_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("surf_list", value, "[Surface]")
        self._surf_list = value

    surf_list = property(
        fget=_get_surf_list,
        fset=_set_surf_list,
        doc=u"""List of surface to draw the Hole. Surfaces must be ordered in trigo order, label must contain HoleMagnet for Magnet and Hole for holes

        :Type: [Surface]
        """,
    )

    def _get_magnet_dict(self):
        """getter of magnet_dict"""
        if self._magnet_dict is not None:
            for key, obj in self._magnet_dict.items():
                if obj is not None:
                    obj.parent = self
        return self._magnet_dict

    def _set_magnet_dict(self, value):
        """setter of magnet_dict"""
        if type(value) is dict:
            for key, obj in value.items():
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "magnet_dict"
                    )
                    value[key] = class_obj(init_dict=obj)
        if type(value) is int and value == -1:
            value = dict()
        check_var("magnet_dict", value, "{Magnet}")
        self._magnet_dict = value

    magnet_dict = property(
        fget=_get_magnet_dict,
        fset=_set_magnet_dict,
        doc=u"""dictionnary with the magnet for the Hole (None to remove magnet, key should be magnet_X)

        :Type: {Magnet}
        """,
    )



# -*- coding: utf-8 -*-
from numpy import pi


def comp_alpha(self):
    """Compute the magnets angle of the slot type 53

    Parameters
    ----------
    self : HoleM53
        a HoleM53 object

    Returns
    -------
    alpha: float
        magnet angle [rad]
    """
    return pi / 2 - self.W4



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Optimization/OptiProblem.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Optimization/OptiProblem
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ntpath import basename
from os.path import isfile
from ._check import CheckTypeError
import numpy as np
import random
from ._check import InitUnKnowClassError
from .Simulation import Simulation
from .OptiDesignVar import OptiDesignVar
from .OptiObjective import OptiObjective
from .OptiConstraint import OptiConstraint
from .DataKeeper import DataKeeper


class OptiProblem(FrozenClass):
    """Multi-objectives optimization problem with some constraints"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        simu=-1,
        design_var=-1,
        obj_func=-1,
        eval_func=None,
        constraint=-1,
        preprocessing=None,
        datakeeper_list=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "simu" in list(init_dict.keys()):
                simu = init_dict["simu"]
            if "design_var" in list(init_dict.keys()):
                design_var = init_dict["design_var"]
            if "obj_func" in list(init_dict.keys()):
                obj_func = init_dict["obj_func"]
            if "eval_func" in list(init_dict.keys()):
                eval_func = init_dict["eval_func"]
            if "constraint" in list(init_dict.keys()):
                constraint = init_dict["constraint"]
            if "preprocessing" in list(init_dict.keys()):
                preprocessing = init_dict["preprocessing"]
            if "datakeeper_list" in list(init_dict.keys()):
                datakeeper_list = init_dict["datakeeper_list"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.simu = simu
        self.design_var = design_var
        self.obj_func = obj_func
        self.eval_func = eval_func
        self.constraint = constraint
        self.preprocessing = preprocessing
        self.datakeeper_list = datakeeper_list

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OptiProblem_str = ""
        if self.parent is None:
            OptiProblem_str += "parent = None " + linesep
        else:
            OptiProblem_str += (
                "parent = " + str(type(self.parent)) + " object" + linesep
            )
        if self.simu is not None:
            tmp = self.simu.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            OptiProblem_str += "simu = " + tmp
        else:
            OptiProblem_str += "simu = None" + linesep + linesep
        if len(self.design_var) == 0:
            OptiProblem_str += "design_var = []" + linesep
        for ii in range(len(self.design_var)):
            tmp = (
                self.design_var[ii].__str__().replace(linesep, linesep + "\t") + linesep
            )
            OptiProblem_str += "design_var[" + str(ii) + "] =" + tmp + linesep + linesep
        if len(self.obj_func) == 0:
            OptiProblem_str += "obj_func = []" + linesep
        for ii in range(len(self.obj_func)):
            tmp = self.obj_func[ii].__str__().replace(linesep, linesep + "\t") + linesep
            OptiProblem_str += "obj_func[" + str(ii) + "] =" + tmp + linesep + linesep
        if self._eval_func_str is not None:
            OptiProblem_str += "eval_func = " + self._eval_func_str + linesep
        elif self._eval_func_func is not None:
            OptiProblem_str += "eval_func = " + str(self._eval_func_func) + linesep
        else:
            OptiProblem_str += "eval_func = None" + linesep + linesep
        if len(self.constraint) == 0:
            OptiProblem_str += "constraint = []" + linesep
        for ii in range(len(self.constraint)):
            tmp = (
                self.constraint[ii].__str__().replace(linesep, linesep + "\t") + linesep
            )
            OptiProblem_str += "constraint[" + str(ii) + "] =" + tmp + linesep + linesep
        if self._preprocessing_str is not None:
            OptiProblem_str += "preprocessing = " + self._preprocessing_str + linesep
        elif self._preprocessing_func is not None:
            OptiProblem_str += (
                "preprocessing = " + str(self._preprocessing_func) + linesep
            )
        else:
            OptiProblem_str += "preprocessing = None" + linesep + linesep
        if len(self.datakeeper_list) == 0:
            OptiProblem_str += "datakeeper_list = []" + linesep
        for ii in range(len(self.datakeeper_list)):
            tmp = (
                self.datakeeper_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            OptiProblem_str += (
                "datakeeper_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        return OptiProblem_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.simu != self.simu:
            return False
        if other.design_var != self.design_var:
            return False
        if other.obj_func != self.obj_func:
            return False
        if other._eval_func_str != self._eval_func_str:
            return False
        if other.constraint != self.constraint:
            return False
        if other._preprocessing_str != self._preprocessing_str:
            return False
        if other.datakeeper_list != self.datakeeper_list:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.simu is None and self.simu is not None) or (
            other.simu is not None and self.simu is None
        ):
            diff_list.append(name + ".simu None mismatch")
        elif self.simu is not None:
            diff_list.extend(self.simu.compare(other.simu, name=name + ".simu"))
        if (other.design_var is None and self.design_var is not None) or (
            other.design_var is not None and self.design_var is None
        ):
            diff_list.append(name + ".design_var None mismatch")
        elif self.design_var is None:
            pass
        elif len(other.design_var) != len(self.design_var):
            diff_list.append("len(" + name + ".design_var)")
        else:
            for ii in range(len(other.design_var)):
                diff_list.extend(
                    self.design_var[ii].compare(
                        other.design_var[ii], name=name + ".design_var[" + str(ii) + "]"
                    )
                )
        if (other.obj_func is None and self.obj_func is not None) or (
            other.obj_func is not None and self.obj_func is None
        ):
            diff_list.append(name + ".obj_func None mismatch")
        elif self.obj_func is None:
            pass
        elif len(other.obj_func) != len(self.obj_func):
            diff_list.append("len(" + name + ".obj_func)")
        else:
            for ii in range(len(other.obj_func)):
                diff_list.extend(
                    self.obj_func[ii].compare(
                        other.obj_func[ii], name=name + ".obj_func[" + str(ii) + "]"
                    )
                )
        if other._eval_func_str != self._eval_func_str:
            diff_list.append(name + ".eval_func")
        if (other.constraint is None and self.constraint is not None) or (
            other.constraint is not None and self.constraint is None
        ):
            diff_list.append(name + ".constraint None mismatch")
        elif self.constraint is None:
            pass
        elif len(other.constraint) != len(self.constraint):
            diff_list.append("len(" + name + ".constraint)")
        else:
            for ii in range(len(other.constraint)):
                diff_list.extend(
                    self.constraint[ii].compare(
                        other.constraint[ii], name=name + ".constraint[" + str(ii) + "]"
                    )
                )
        if other._preprocessing_str != self._preprocessing_str:
            diff_list.append(name + ".preprocessing")
        if (other.datakeeper_list is None and self.datakeeper_list is not None) or (
            other.datakeeper_list is not None and self.datakeeper_list is None
        ):
            diff_list.append(name + ".datakeeper_list None mismatch")
        elif self.datakeeper_list is None:
            pass
        elif len(other.datakeeper_list) != len(self.datakeeper_list):
            diff_list.append("len(" + name + ".datakeeper_list)")
        else:
            for ii in range(len(other.datakeeper_list)):
                diff_list.extend(
                    self.datakeeper_list[ii].compare(
                        other.datakeeper_list[ii],
                        name=name + ".datakeeper_list[" + str(ii) + "]",
                    )
                )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.simu)
        if self.design_var is not None:
            for value in self.design_var:
                S += getsizeof(value)
        if self.obj_func is not None:
            for value in self.obj_func:
                S += getsizeof(value)
        S += getsizeof(self._eval_func_str)
        if self.constraint is not None:
            for value in self.constraint:
                S += getsizeof(value)
        S += getsizeof(self._preprocessing_str)
        if self.datakeeper_list is not None:
            for value in self.datakeeper_list:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        OptiProblem_dict = dict()
        if self.simu is None:
            OptiProblem_dict["simu"] = None
        else:
            OptiProblem_dict["simu"] = self.simu.as_dict(**kwargs)
        if self.design_var is None:
            OptiProblem_dict["design_var"] = None
        else:
            OptiProblem_dict["design_var"] = list()
            for obj in self.design_var:
                if obj is not None:
                    OptiProblem_dict["design_var"].append(obj.as_dict(**kwargs))
                else:
                    OptiProblem_dict["design_var"].append(None)
        if self.obj_func is None:
            OptiProblem_dict["obj_func"] = None
        else:
            OptiProblem_dict["obj_func"] = list()
            for obj in self.obj_func:
                if obj is not None:
                    OptiProblem_dict["obj_func"].append(obj.as_dict(**kwargs))
                else:
                    OptiProblem_dict["obj_func"].append(None)
        if self._eval_func_str is not None:
            OptiProblem_dict["eval_func"] = self._eval_func_str
        elif "keep_function" in kwargs and kwargs["keep_function"]:
            OptiProblem_dict["eval_func"] = self.eval_func
        else:
            OptiProblem_dict["eval_func"] = None
            if self.eval_func is not None:
                self.get_logger().warning(
                    "OptiProblem.as_dict(): "
                    + f"Function {self.eval_func.__name__} is not serializable "
                    + "and will be converted to None."
                )
        if self.constraint is None:
            OptiProblem_dict["constraint"] = None
        else:
            OptiProblem_dict["constraint"] = list()
            for obj in self.constraint:
                if obj is not None:
                    OptiProblem_dict["constraint"].append(obj.as_dict(**kwargs))
                else:
                    OptiProblem_dict["constraint"].append(None)
        if self._preprocessing_str is not None:
            OptiProblem_dict["preprocessing"] = self._preprocessing_str
        elif "keep_function" in kwargs and kwargs["keep_function"]:
            OptiProblem_dict["preprocessing"] = self.preprocessing
        else:
            OptiProblem_dict["preprocessing"] = None
            if self.preprocessing is not None:
                self.get_logger().warning(
                    "OptiProblem.as_dict(): "
                    + f"Function {self.preprocessing.__name__} is not serializable "
                    + "and will be converted to None."
                )
        if self.datakeeper_list is None:
            OptiProblem_dict["datakeeper_list"] = None
        else:
            OptiProblem_dict["datakeeper_list"] = list()
            for obj in self.datakeeper_list:
                if obj is not None:
                    OptiProblem_dict["datakeeper_list"].append(obj.as_dict(**kwargs))
                else:
                    OptiProblem_dict["datakeeper_list"].append(None)
        # The class name is added to the dict for deserialisation purpose
        OptiProblem_dict["__class__"] = "OptiProblem"
        return OptiProblem_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.simu is not None:
            self.simu._set_None()
        self.design_var = None
        self.obj_func = None
        self.eval_func = None
        self.constraint = None
        self.preprocessing = None
        self.datakeeper_list = None

    def _get_simu(self):
        """getter of simu"""
        return self._simu

    def _set_simu(self, value):
        """setter of simu"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "simu")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Simulation()
        check_var("simu", value, "Simulation")
        self._simu = value

        if self._simu is not None:
            self._simu.parent = self

    simu = property(
        fget=_get_simu,
        fset=_set_simu,
        doc="""Default simulation

        :Type: Simulation
        """,
    )

    def _get_design_var(self):
        """getter of design_var"""
        if self._design_var is not None:
            for obj in self._design_var:
                if obj is not None:
                    obj.parent = self
        return self._design_var

    def _set_design_var(self, value):
        """setter of design_var"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "design_var"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("design_var", value, "[OptiDesignVar]")
        self._design_var = value

    design_var = property(
        fget=_get_design_var,
        fset=_set_design_var,
        doc="""List of design variables

        :Type: [OptiDesignVar]
        """,
    )

    def _get_obj_func(self):
        """getter of obj_func"""
        if self._obj_func is not None:
            for obj in self._obj_func:
                if obj is not None:
                    obj.parent = self
        return self._obj_func

    def _set_obj_func(self, value):
        """setter of obj_func"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "obj_func"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("obj_func", value, "[OptiObjective]")
        self._obj_func = value

    obj_func = property(
        fget=_get_obj_func,
        fset=_set_obj_func,
        doc="""List of objective functions

        :Type: [OptiObjective]
        """,
    )

    def _get_eval_func(self):
        """getter of eval_func"""
        return self._eval_func_func

    def _set_eval_func(self, value):
        """setter of eval_func"""
        if value is None:
            self._eval_func_str = None
            self._eval_func_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._eval_func_str = value
            self._eval_func_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._eval_func_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._eval_func_func = eval(basename(value[:-3]))
        elif callable(value):
            self._eval_func_str = None
            self._eval_func_func = value
        else:
            raise CheckTypeError(
                "For property eval_func Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    eval_func = property(
        fget=_get_eval_func,
        fset=_set_eval_func,
        doc="""Function to evaluate before computing obj function and constraints

        :Type: function
        """,
    )

    def _get_constraint(self):
        """getter of constraint"""
        if self._constraint is not None:
            for obj in self._constraint:
                if obj is not None:
                    obj.parent = self
        return self._constraint

    def _set_constraint(self, value):
        """setter of constraint"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "constraint"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("constraint", value, "[OptiConstraint]")
        self._constraint = value

    constraint = property(
        fget=_get_constraint,
        fset=_set_constraint,
        doc="""List containing the constraints 

        :Type: [OptiConstraint]
        """,
    )

    def _get_preprocessing(self):
        """getter of preprocessing"""
        return self._preprocessing_func

    def _set_preprocessing(self, value):
        """setter of preprocessing"""
        if value is None:
            self._preprocessing_str = None
            self._preprocessing_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._preprocessing_str = value
            self._preprocessing_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._preprocessing_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._preprocessing_func = eval(basename(value[:-3]))
        elif callable(value):
            self._preprocessing_str = None
            self._preprocessing_func = value
        else:
            raise CheckTypeError(
                "For property preprocessing Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    preprocessing = property(
        fget=_get_preprocessing,
        fset=_set_preprocessing,
        doc="""Function to execute a preprocessing on the simulation right before it is run.

        :Type: function
        """,
    )

    def _get_datakeeper_list(self):
        """getter of datakeeper_list"""
        if self._datakeeper_list is not None:
            for obj in self._datakeeper_list:
                if obj is not None:
                    obj.parent = self
        return self._datakeeper_list

    def _set_datakeeper_list(self, value):
        """setter of datakeeper_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "datakeeper_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("datakeeper_list", value, "[DataKeeper]")
        self._datakeeper_list = value

    datakeeper_list = property(
        fget=_get_datakeeper_list,
        fset=_set_datakeeper_list,
        doc="""List of DataKeepers to run on every output

        :Type: [DataKeeper]
        """,
    )



# -*- coding: utf-8 -*-

from numpy import arcsin, sin, pi


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotW26
        A SlotW26 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    # 2 half circle
    S1 = 0.5 * pi * self.R1 ** 2
    S2 = 0.5 * pi * self.R2 ** 2
    # Trapeze
    S3 = self.H1 * (2 * self.R1 + 2 * self.R2) / 2

    # Angle of the arc (P2,C1,P7)
    alpha2 = 2 * arcsin(self.W0 / (2.0 * self.R1))

    # Surface of arc (P2,C1,P7) in the isthmus
    Sarc = (self.R1 ** 2.0) / 2.0 * (alpha2 - sin(alpha2))

    return S1 + S2 + S3 - Sarc



# -*- coding: utf-8 -*-


from PySide2.QtCore import QSize
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QDialog, QMessageBox, QSpinBox, QTableWidgetItem

from ......Classes.Lamination import Lamination
from ......Classes.VentilationCirc import VentilationCirc
from ......GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.Ui_DAVDuct import Ui_DAVDuct
from ......GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.WVent.WVent import WVent


class DAVDuct(Ui_DAVDuct, QDialog):
    """Dialog to setup the ventilations"""

    def __init__(self, lamination):
        """Initialize the widget according the current lamination

        Parameters
        ----------
        self : DAVDuct
            A DAVDuct widget
        lam : Lamination
            current lamination to edit
        """
        # Build the interface according to the .ui file
        QDialog.__init__(self)
        self.setupUi(self)

        self.obj = lamination  # Current object
        self.lam = Lamination(
            init_dict=Lamination.as_dict(lamination)
        )  # Copy to modify

        # Init the GUI
        if len(self.lam.axial_vent) == 0:  # No vent => init circle
            self.lam.axial_vent.append(VentilationCirc())
            self.lam.axial_vent[0]._set_None()

        self.tab_vent.clear()
        for vent in self.lam.axial_vent:
            self.s_add(vent)
        self.tab_vent.setCurrentIndex(0)

        # Set Help URL
        self.b_help.hide()

        self.b_new.clicked.connect(self.s_add)
        self.b_remove.clicked.connect(self.s_remove)
        self.b_plot.clicked.connect(self.plot)
        self.b_cancel.clicked.connect(self.reject)
        self.b_ok.clicked.connect(self.valid_vent)

    def s_add(self, vent=False):
        """Signal to add a new hole

        Parameters
        ----------
        self : DAVDuct
            A DAVDuct widget
        vent : Ventilation
            The ventilation to init the GUI with
        """
        # Create a new hole if needed
        if type(vent) is bool:
            # Default Hole is Circular
            vent_obj = VentilationCirc()
            vent_obj._set_None()
            self.lam.axial_vent.append(vent_obj)
            index = len(self.lam.axial_vent) - 1
        else:
            index = self.lam.axial_vent.index(vent)
        tab = WVent(self.lam, index=index)
        self.tab_vent.addTab(tab, "Vent " + str(index + 1))

    def s_remove(self):
        """Signal to remove the last hole

        Parameters
        ----------
        self : DAVDuct
            a DAVDuct object
        """
        if len(self.lam.axial_vent) > 1:
            self.tab_vent.removeTab(len(self.lam.axial_vent) - 1)
            self.lam.axial_vent.pop(-1)

    def plot(self):
        """Plot the ventilation ducts according to the table

        Parameters
        ----------
        self : DAVDuct
            a DAVDuct object
        """
        # We have to make sure the hole is right before trying to plot it
        error = self.check()

        if error:  # Error => Display it
            QMessageBox().critical(self, self.tr("Error"), error)
        else:  # No error => Plot the hole (No winding for LamSquirrelCage)
            self.lam.plot()

    def valid_vent(self):
        """Validate the new ventilation and update the lamination

        Parameters
        ----------
        self : DAVDuct
            a DAVDuct object
        """
        error = self.check()

        if error:  # Error => Display it
            QMessageBox().critical(self, self.tr("Error"), error)
        else:
            self.obj.axial_vent = self.lam.axial_vent
            self.accept()

    def check(self):
        """Check that all the ventilation are correctly set

        Parameters
        ----------
        self : DAVDuct
            a DAVDuct object
        """
        for ii in range(self.tab_vent.count()):
            error = self.tab_vent.widget(ii).check()
            if error is not None:
                return "Vent " + str(ii + 1) + ": " + error
        return None
